
# ============================================================
# ClientDatabase.py
# provides methods for manipulating client data to all apps.
# ------------------------------------------------------------
# Copyright (C) 2004 the Kumula Team.
# Licensed under the LGPL.
# ============================================================

import kumula
from qt import QString, Qt
from qtsql import QSqlQuery,QSqlDatabase
import Database

# ------------------------------------------------------------


class KClient(object):

    def __init__(self, anID):
        self.clientID = anID
        quClient = kumula.DB.doSelect("clients", ("name",), "", {"id":self.clientID}, 1)
        while quClient.next():
            self.clientName = quClient.value(0).toString()





class KClientInterface(object):

    def __init__(self, aDB):
        self.ClientDB = aDB


# ------------------------------------------------------------
# deleteClient
# deletes the client with id "aClientID"


    def deleteClient(self, aClientID):
        if self.getClientName == "":
            return -1
        self.ClientDB.doDelete("clients_2group", {"clientid":aClientID})
        self.ClientDB.doDelete("clients", {"id":aClientID})


# ------------------------------------------------------------
# deleteGroup
# deletes the group with id "aGroupID"


    def deleteGroup(self, aGroupID):
        if self.getGroupName(aGroupID) == "":
            return -1
        self.ClientDB.doDelete("clients_2group", {"groupid":"%i" % aGroupID})
        self.ClientDB.doDelete("clients_groups", {"id":"%i" % aGroupID})


# ------------------------------------------------------------
# getClientName
# return the name of the client with id "aClientID"


    def getClientName(self, aClientID):
        myName = QString("")
        quClient = self.ClientDB.doSelect("clients", ("name",), "", {"id":aClientID}, 1)
        while quClient.next():
            myName = quClient.value(0).toString()
        return myName

    def getClientDescription(self, aClientID):
        myDescr = QString("")
        quClient = self.ClientDB.doSelect("clients", ("description",), "", {"id":aClientID}, 1)
        while quClient.next():
            myDescr = quClient.value(0).toString()
        return myDescr


# ------------------------------------------------------------
# getClientsPerGroup
# return a Python tuple with the IDs of the clients in the group "aGroupID"


    def getClientsPerGroup(self, aGroupID):
        # if aGroupID is -1, all clients from any group will be returned
        if aGroupID == -1:
            quClients = self.ClientDB.doSelect("clients", ("id",), "id")
        else:
            quClients = self.ClientDB.doSelect("clients_2group", ("clientid",), "clientid", {"groupid":"%i" % aGroupID})
        rTuple = ()
        while quClients.next():
            rTuple = rTuple + (quClients.value(0).toString(),)
        return rTuple


# ------------------------------------------------------------
# getGroups
# returns all group names


    def getGroups(self):
        rTuple = ()
        quGroups = self.ClientDB.doSelect("clients_groups", ("id","name","color"), "name", {"status":"0"})
        while quGroups.next():
            eTuple = (quGroups.value(0).toInt(), quGroups.value(1).toString(), quGroups.value(2).toString())
            rTuple = rTuple + (eTuple,)
        return rTuple


# ------------------------------------------------------------
# getGroupFolderColor
# return the color of the group's folder... just a gimmick... ;-)


    def getGroupFolderColor(self, aGroupName):
        # default color
        myColor = QString("blue")
        quGroup = self.ClientDB.doSelect("clients_groups", ("color",), "", {"name":aGroupName}, 1)
        while quGroup.next():
            myColor = quGroup.value(0).toString()
        return myColor


# ------------------------------------------------------------
# getGroupName
# return the name of the group with id "aGroupID"


    def getGroupName(self, aGroupID):
        myName = QString("")
        quGroup = self.ClientDB.doSelect("clients_groups", ("name",), "", {"id":"%i" % aGroupID}, 1)
        while quGroup.next():
            myName = quGroup.value(0).toString()
        return myName


# ------------------------------------------------------------
# getGroupID
# return the ID of the group with its name


    def getGroupID(self, aGroupName):
        # a return value of -1 means, no group with this name exists
        myID = -1
        quGroup = self.ClientDB.doSelect("clients_groups", ("id",), "", {"name":aGroupName}, 1)
        while quGroup.next():
            myID = quGroup.value(0).toInt()
        return myID


# ------------------------------------------------------------
# newClient
# creates a new client in the database


    def newClient(self, aID, aName):
        if (aID == ""):
            return -1
        if (aName == ""):
            return -1
        if (self.getClientName(aID) != ""):       # client with this ID already exists
            return -2
        self.ClientDB.doInsert("clients", {"id":aID, "name":aName, "status":"0"})
        return 0


    def setClientDescription(self, anID, aDescription):
        self.ClientDB.doUpdate("clients", {"description":aDescription}, {"id":anID})



# ------------------------------------------------------------
# setClientName
# changes the name of a client


    def setClientName(self, aID, aName):
        if (aID == ""):
            return -1
        if (aName == ""):
            return -1
        if (self.getClientName(aID) == ""):       # this client doesn't exist
            return -2
        self.ClientDB.doUpdate("clients", {"name":aName}, {"id":aID}, 1)
        return 0


# ------------------------------------------------------------
# newGroup
# creates a new group in the database


    def newGroup(self, aName, aColor="blue"):
        if (aName == ""):
            return -1
        if (self.getGroupID(aName) != -1):        # group with this name already exist
            return -2
        self.ClientDB.doInsert("clients_groups", {"name":aName, "color":aColor, "status":"0"})
        return 0


# ------------------------------------------------------------
# setGroupName
# changes the name of a group


    def setGroupName(self, aID, aName):
        if (aName == ""):
            return -1
        if (self.getGroupName(aID) == ""):        # group doesn't exist
            return -2
        if (self.getGroupID(aName) != -1):        # group with this name already exist
            return -1
        self.ClientDB.doUpdate("clients_groups", {"name":aName}, {"id":"%i" % aID}, 1)
        return 0


# ------------------------------------------------------------
# setGroupFolderColor
# changes the color of a group's folder


    def setGroupFolderColor(self, aName, aColor):
        if (aName == ""):
            return -1
        if (aColor == ""):
            return -1
        myID = self.getGroupID(aName)
        if (myID == -1):      # this group doesn't exist
            return -1
        self.ClientDB.doUpdate("clients_groups", {"color":aColor}, {"id":"%i" % myID}, 1)
        return 0


# ------------------------------------------------------------
# getGroupsPerClient
# in which groups is the client "aClientID" member of?


    def getGroupsPerClient(self, aClientID):
        rTuple = ()
        quGroups = self.ClientDB.doSelect("clients_2group", ("groupid",), "", {"clientid":aClientID})
        while quGroups.next():
            rTuple = rTuple + (quGroups.value(0).toInt(),)
        return rTuple


# ------------------------------------------------------------
# getGroupNamesPerClient
# same as above, with group names instead of IDs


    def getGroupNamesPerClient(self, aClientID):
        myGroups = self.getGroupsPerClient(aClientID)
        rTuple = ()
        for myValue in myGroups:
            rTuple = rTuple + (self.getGroupName(myValue),)
        return rTuple


# ------------------------------------------------------------
# setGroupsPerClient
# assign a client to multiple groups


    def setGroupsPerClient(self, aClientID, aGroupTuple):
        # delete existing assignments from this client
        self.ClientDB.doDelete("clients_2group", {"clientid":aClientID})
        # set the new assignments
        for myGroup in aGroupTuple:
            self.ClientDB.doInsert("clients_2group", {"clientid":aClientID, "groupid":"%i" % myGroup})


    def getClientStatus(self, aClientID):
        myStatus = -1
        quClient = self.ClientDB.doSelect("clients", ("status",), "", {"id":aClientID}, 1)
        while quClient.next():
            myStatus = quClient.value(0).toInt()
        return myStatus

    def setClientStatus(self, aClientID, status):
        quClient = self.ClientDB.doUpdate("clients", {"status":"%i" % status}, {"id":aClientID}, 1)




    def getAddressFromClient(self, aClientID):
        quAddress = self.ClientDB.doSelect("clients_addresses", ("zip","city","district","address"), "id", {"clientid":aClientID}, 1)
        while quAddress.next():
            myZip      = quAddress.value(0).toString()
            myCity     = quAddress.value(1).toString()
            myDistrict = quAddress.value(2).toString()
            myAddress  = quAddress.value(3).toString()
            myReturnString = QString(myZip)
            myReturnString.append(" ")
            myReturnString.append(myCity)
            if (myDistrict != ""):
                myReturnString.append("-")
                myReturnString.append(myDistrict)
            myReturnString.append(", ")
            myReturnString.append(myAddress)
#            myTuple    = (myZip, myCity, myDistrict, myAddress)
            return myReturnString



    def isUserRelated(self, aClientID, aUserName):
        related = 0
        quRel = self.ClientDB.doSelect("clients_2user", ("clientid",), "", {"clientid":aClientID,"userid":aUserName}, 1)
        while quRel.next():
            related = 1
        return related

    def setUserRelation(self, aClientID, aUserName, rel):
        if (rel == 0):
            # relation has to be deleted
            self.ClientDB.doDelete("clients_2user", {"clientid":aClientID,"userid":aUserName}, 1)
        else:
            # relation has to be set
            self.ClientDB.doInsert("clients_2user", {"clientid":aClientID,"userid":aUserName})






