# ============================================================
# ClientSelectDlg.py
# provides an dialog where the user can select a client.
# ------------------------------------------------------------
# Copyright (C) 2004 the Kumula Team.
# Licensed under the LGPL.
# ============================================================

import os
from qt import *
import kumula
from SpecialItems import KIconViewItemWithID
from SideBar import KSideBar
from SpecialItems import KColoredListViewItem

# ------------------------------------------------------------


class KClientSelectDlg(QDialog):

    def __init__(self,parent = None,name = None,modal = 0,fl = 0):
    
        QDialog.__init__(self,parent,name,modal,fl)
        if not name:
            self.setName("KClientSelectDlg")
        self.setModal(1)
        origProgName = kumula.ProgramName
        kumula.ProgramName = QString("clients")
        self.setCaption("Please select a client")
        self.setIcon(kumula.getProgramIcon())
        kumula.ProgramName = origProgName


        ClientSelectDlgLayout = QVBoxLayout(self,11,6,"ClientSelectDlgLayout")
        InfoLayout = QHBoxLayout(None,0,6,"InfoLayout")

        self.listGroups = KSideBar(self, "listGroups")
        InfoLayout.addWidget(self.listGroups)

        self.listClients = QListView(self,"listClients")
        self.listClients.addColumn("ID")
        self.listClients.addColumn("Name")
        self.listClients.setAllColumnsShowFocus(1)
        self.listClients.setResizeMode(QListView.LastColumn)
        InfoLayout.addWidget(self.listClients)
        ClientSelectDlgLayout.addLayout(InfoLayout)

        self.separatorLine = QFrame(self,"separatorLine")
        self.separatorLine.setFrameShape(QFrame.HLine)
        self.separatorLine.setFrameShadow(QFrame.Sunken)
        self.separatorLine.setFrameShape(QFrame.HLine)
        ClientSelectDlgLayout.addWidget(self.separatorLine)

        ButtonLayout = QHBoxLayout(None,0,6,"ButtonLayout")

        self.sideBarAttrMenu = QPopupMenu(self)
        itemCount = 0
        quAttributes = kumula.DB.doSelect("clients_attributes", ("id","title"), "title")
        while quAttributes.next():
            myAttrID    = quAttributes.value(0).toInt()
            myAttrTitle = quAttributes.value(1).toString()
            myItemID = self.sideBarAttrMenu.insertItem(myAttrTitle, self.handleViewSideBarAttribute)
            self.sideBarAttrMenu.setItemParameter(myItemID, myAttrID)
            itemCount = itemCount + 1
        if (itemCount == 0):
            myItemID = self.sideBarAttrMenu.insertItem("no attributes")
            self.sideBarAttrMenu.setItemParameter(myItemID, -1)
            self.sideBarAttrMenu.setItemEnabled(myItemID, 0)


        self.settingsMenu = QPopupMenu(self)
        self.actViewRelatedClients   = QAction(self, "actViewRelatedClients")
        self.actViewPotentialClients = QAction(self, "actViewPotentialClients")
        self.actViewLostClients      = QAction(self, "actViewLostClients")
        self.actViewRelatedClients.setMenuText("...only related clients")
        self.actViewPotentialClients.setMenuText("...potential clients")
        self.actViewLostClients.setMenuText("...lost clients")
        self.connect(self.actViewRelatedClients,   SIGNAL("activated()"), self.handleViewRelatedClients)
        self.connect(self.actViewPotentialClients, SIGNAL("activated()"), self.handleViewPotentialClients)
        self.connect(self.actViewLostClients,      SIGNAL("activated()"), self.handleViewLostClients)
        self.actViewRelatedClients.addTo(self.settingsMenu)
        self.settingsMenu.insertSeparator()
        self.actViewPotentialClients.addTo(self.settingsMenu)
        self.actViewLostClients.addTo(self.settingsMenu)
        self.settingsMenu.insertSeparator()
        self.settingsMenu.insertItem(QString("SideBar attribute"), self.sideBarAttrMenu, 9)

        self.btnSettings = QPushButton(self,"btnSettings")
        self.btnSettings.setText("&Show...")
        self.btnSettings.setPopup(self.settingsMenu)
        ButtonLayout.addWidget(self.btnSettings)

        spacer = QSpacerItem(40,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        ButtonLayout.addItem(spacer)

        self.btnOK = QPushButton(self,"btnOK")
        self.btnOK.setText("&Use this client")
        ButtonLayout.addWidget(self.btnOK)
        self.btnCancel = QPushButton(self,"btnCancel")
        self.btnCancel.setText("&Close")
        ButtonLayout.addWidget(self.btnCancel)
        ClientSelectDlgLayout.addLayout(ButtonLayout)

        # initialize Settings
        self.viewRelatedClients   = kumula.GlobalSettings.readInt("clientsView", "relatedClients", 0)
        self.viewPotentialClients = kumula.GlobalSettings.readInt("clientsView", "potentialClients", 1)
        self.viewLostClients      = kumula.GlobalSettings.readInt("clientsView", "lostClients", 0)
        self.colClient            = kumula.GlobalSettings.readString("colors", "Client", "#000000")
        self.colLostClient        = kumula.GlobalSettings.readString("colors", "LostClient", "#808080")
        self.colPotential         = kumula.GlobalSettings.readString("colors", "Potential", "#800000")
        self.colLostPotential     = kumula.GlobalSettings.readString("colors", "LostPotential", "#FF8080")
        self.curAttribute         = kumula.GlobalSettings.readInt("clientsView", "lastAttribute",  -1)

        self.actViewRelatedClients.setToggleAction(1)
        self.actViewRelatedClients.setOn(self.viewRelatedClients)
        self.actViewPotentialClients.setToggleAction(1)
        self.actViewPotentialClients.setOn(self.viewPotentialClients)
        self.actViewLostClients.setToggleAction(1)
        self.actViewLostClients.setOn(self.viewLostClients)

        myWidth  = kumula.GlobalSettings.readInt("ClientSelectDlg", "dlgWidth",  360)
        myHeight = kumula.GlobalSettings.readInt("ClientSelectDlg", "dlgHeight", 400)
        self.resize(QSize(myWidth,myHeight).expandedTo(self.minimumSizeHint()))
        self.clearWState(Qt.WState_Polished)

        self.connect(self.listGroups,PYSIGNAL("selectionChanged()"),self.changedGroup)
        self.connect(self.btnOK,SIGNAL("clicked()"),self,SLOT("accept()"))
        self.connect(self.btnCancel,SIGNAL("clicked()"),self,SLOT("reject()"))
        self.updateSideBar()
        self.listClients.setFocus()



    def handleViewRelatedClients(self):
        self.viewRelatedClients = int(self.actViewRelatedClients.isOn())
        kumula.GlobalSettings.writeInt("clientsView", "relatedClients", self.viewRelatedClients)
        self.updateClientList()

    def handleViewPotentialClients(self):
        self.viewPotentialClients = int(self.actViewPotentialClients.isOn())
        kumula.GlobalSettings.writeInt("clientsView", "potentialClients", self.viewPotentialClients)
        self.updateClientList()

    def handleViewLostClients(self):
        self.viewLostClients = int(self.actViewLostClients.isOn())
        kumula.GlobalSettings.writeInt("clientsView", "lostClients", self.viewLostClients)
        self.updateClientList()


    def handleViewSideBarAttribute(self, attrID):
        self.curAttribute = attrID
        kumula.GlobalSettings.writeInt("clientsView", "lastAttribute",  attrID)
        self.updateSideBar()



    def updateGroupList(self):
        self.disconnect(self.listGroups, PYSIGNAL("selectionChanged()"), self.changedGroup)
        self.listGroups.clear()

        # save the original program name, because for the
        # correct folder icons we need the "graphics/clients" path
        origProgName = kumula.ProgramName
        kumula.ProgramName = QString("clients")

        # Item for ALL clients
        self.listGroups.insertButtonAuto("All clients", "folder_global", -1)
        # Items for user-defined groups
        userGroups = kumula.ClientIntf.getGroups()
        for grTuple in userGroups:
            grID    = grTuple[0]
            grName  = grTuple[1]
            grColor = grTuple[2]
            myIcon  = QString("folder_")
            myIcon.append(grColor)
            self.listGroups.insertButtonAuto(grName, myIcon, grID)
        self.listGroups.insertSpacer()

        # restore the original program name
        kumula.ProgramName = origProgName

        self.connect(self.listGroups, PYSIGNAL("selectionChanged()"), self.changedGroup)
        self.listGroups.selectButton(-1)


    def updateSideBar(self):
        self.disconnect(self.listGroups,  PYSIGNAL("selectionChanged()"), self.changedGroup)
        self.listGroups.clear()

        # save the original program name, because for the
        # correct folder icons we need the "graphics/clients" path
        origProgName = kumula.ProgramName
        kumula.ProgramName = QString("clients")

        # Item for ALL clients
        self.listGroups.insertButtonAuto("All clients", "folder_global", -1)
        quOptions = kumula.DB.doSelect("clients_attroptions", ("id","title","icon"), "title", {"attrid":"%i" % self.curAttribute})
        while quOptions.next():
            optID    = quOptions.value(0).toInt()
            optTitle = quOptions.value(1).toString()
            optIcon  = quOptions.value(2).toString()
            optIcon.prepend("attricons/")
            optIcon.append(".png")
            self.listGroups.insertButton(optTitle, QPixmap(QString(os.path.join(kumula.getIconDir(), optIcon.ascii()))), optID)
        self.listGroups.insertSpacer()

        # restore the original program name
        kumula.ProgramName = origProgName

        self.connect(self.listGroups,  PYSIGNAL("selectionChanged()"), self.changedGroup)
        self.listGroups.selectButton(-1)



#    def updateClientList(self):
#        self.listClients.clear()
#        myGroupID = self.listGroups.currentButton()
#        myClients = kumula.ClientIntf.getClientsPerGroup(myGroupID)
#        count = 0
#        for clID in myClients:
#            clName = kumula.ClientIntf.getClientName(clID)
#            item   = QListViewItem(self.listClients)
#            item.setText(0, clID)
#            item.setText(1, clName)
#            count = count + 1

#        if (count == 0):
#            self.btnOK.setEnabled(0)
#        else:
#            self.btnOK.setEnabled(1)
#            self.listClients.setSelected(self.listClients.firstChild(), 1)


    def changedGroup(self):
        self.updateClientList()


    def updateClientList(self, selectedClient=''):
        # save ID of the selected client, and if possible, select it after updating the list
        if (selectedClient == ''):
            if self.listClients.currentItem():
                selectedClient = self.listClients.currentItem().text(0)
        self.listClients.clear()
        hasSelected = 0
        myGroupID = self.listGroups.currentButton()
        myClients = kumula.ClientIntf.getClientsPerGroup(myGroupID)
        for clID in myClients:
            clStatus = kumula.ClientIntf.getClientStatus(clID)
            visibleItem = 1
            if (self.viewLostClients == 0):
                if (clStatus == 1) or (clStatus == 3):
                    visibleItem = 0
            if (self.viewPotentialClients == 0):
                if (clStatus == 2) or (clStatus == 3):
                    visibleItem = 0
            if (self.viewRelatedClients == 1):
                if (not kumula.ClientIntf.isUserRelated(clID, kumula.UserName)):
                    visibleItem = 0
            if (visibleItem == 1):
                clName = kumula.ClientIntf.getClientName(clID)
                item   = KColoredListViewItem(self.listClients)
                item.setText(0, clID)
                item.setText(1, clName)
                self.colorItem(item, clStatus)
                if (clID == selectedClient):
                    self.listClients.setSelected(item, 1)
                    self.listClients.setCurrentItem(item)
                    hasSelected = 1
        self.btnOK.setEnabled(1)
        if (hasSelected == 0):
            if self.listClients.firstChild():
                self.listClients.setCurrentItem(self.listClients.firstChild())
                self.listClients.setSelected(self.listClients.firstChild(), 1)
            else:
                self.btnOK.setEnabled(0)


    def colorItem(self, item, status):
        if (status == 1):
            item.setColor(self.colLostClient)
        elif (status == 2):
            item.setColor(self.colPotential)
        elif (status == 3):
            item.setColor(self.colLostPotential)
        else:
            item.setColor(self.colClient)


    def __tr(self,s,c = None):
        return qApp.translate("KClientSelectDlg",s,c)
