# ============================================================
# CountryInterface.py
# manages countries, zones and cities
# ------------------------------------------------------------
# Copyright (C) 2003-2005 the Kumula Team.
# Licensed under the LGPL.
# ============================================================

from qt import *

# ------------------------------------------------------------

class KCountryInterface(object):


    def __init__(self, aDB):
        self.DB = aDB


    def getCountries(self, isolevel):
        quCountries = self.DB.doSelect("countries", ("id","name","iso2","iso3"))
        rTuple = ()
        while quCountries.next():
            myID   = QString(quCountries.value(0).toString())
            myName = QString(quCountries.value(1).toString())
            myIso2 = QString(quCountries.value(2).toString())
            myIso3 = QString(quCountries.value(3).toString())
            if (isolevel == 2):
                eTuple = (myID, myName, myIso2)
            elif (isolevel == 3):
                eTuple = (myID, myName, myIso3)
            else:
                eTuple = (myID, myName)
            rTuple = rTuple + (eTuple,)
        return rTuple


    def getCountry(self, aCountryID, isolevel):
        quCountries = self.DB.doSelect("countries", ("name","iso2","iso3"), "", {"id":aCountryID}, 1)
        eTuple = (0, "")
        while quCountries.next():
            myName = QString(quCountries.value(0).toString())
            myIso2 = QString(quCountries.value(1).toString())
            myIso3 = QString(quCountries.value(2).toString())
            if (isolevel == 2):
                eTuple = (aCountryID, myName, myIso2)
            elif (isolevel == 3):
                eTuple = (aCountryID, myName, myIso3)
            else:
                eTuple = (aCountryID, myName)
        return eTuple


    def getZonesOfCountry(self, country):
        quZones = self.DB.doSelect("zones", ("id","name","code"), "", {"countryid":country})
        rTuple = ()
        while quZones.next():
            myID   = QString(quZones.value(0).toString())
            myName = QString(quZones.value(1).toString())
            myCode = QString(quZones.value(2).toString())
            eTuple = (myID, myName, myCode)
            rTuple = rTuple + (eTuple,)
        return rTuple


    def countCountryZones(self, country):
        quZones = self.DB.doSelect("zones", ("id",), "", {"countryid":country})
        zoneCount = 0
        while quZones.next():
            zoneCount = zoneCount + 1
        return zoneCount



    def getZoneName(self, aZoneID):
        quZones = self.DB.doSelect("zones", ("name",), "", {"id":aZoneID}, 1)
        myName = QString("")
        while quZones.next():
            myName = QString(quZones.value(0).toString())
        return myName


    def getCountryFromZone(self, aZoneID, isolevel):
        quZones = self.DB.doSelect("zones", ("countryid",), "", {"id":aZoneID}, 1)
        myCountryID = QString("")
        while quZones.next():
            myCountryID = QString(quZones.value(0).toString())
            quCountries = self.DB.doSelect("countries", ("name","iso2","iso3"), "", {"id":myCountryID}, 1)
            while quCountries.next():
                myName = QString(quCountries.value(0).toString())
                myIso2 = QString(quCountries.value(1).toString())
                myIso3 = QString(quCountries.value(2).toString())
                if (isolevel == 2):
                    eTuple = (myCountryID, myName, myIso2)
                elif (isolevel == 3):
                    eTuple = (myCountryID, myName, myIso3)
                else:
                    eTuple = (myCountryID, myName)
                return eTuple


    def getCitiesFromZone(self, aZip, aZone):
        quCities = self.DB.doSelect("cities", ("zip","name"), "", {"zip":aZip, "zoneid":aZone})
        rTuple = ()
        while quCities.next():
            myZip  = QString(quCities.value(0).toString())
            myName = QString(quCities.value(1).toString())
            eTuple = (myZip, myName)
            rTuple = rTuple + (eTuple,)
        return rTuple


    def getCitiesFromCountry(self, aZip, aCountry):
        zoneCount = self.countCountryZones(aCountry)
        quCities = self.DB.doSelect("cities", ("zoneid","zip","name"), "", {"zip":aZip})
        rTuple = ()
        while quCities.next():
            wantedCity = 0
            if (zoneCount == 0):
                # Country has no zones, the 'zoneid' is the countryid
                if (quCities.value(0).toString() == aCountry):
                    wantedCity = 1
            else:
                # Country has zones, check each zone to be in the country
                quZones = self.DB.doSelect("zones", ("countryid",), "", {"id":quCities.value(0).toString()})
                while quZones.next():
                    if (quZones.value(0).toString() == aCountry):
                        wantedCity = 1
            if (wantedCity):
                eTuple = (quCities.value(1).toString(), quCities.value(2).toString())
                rTuple = rTuple + (eTuple,)
        return rTuple



