# ============================================================
# KumulaMDIWindow.py
# this is a parent class for MDI windows.
# provides the window menu.
# ------------------------------------------------------------
# Copyright (C) 2004 the Kumula Team.
# Licensed under the LGPL.
# ============================================================

import kumula
from qt import *
from KumulaMainWindow import KKumulaMainWindow


# ------------------------------------------------------------


class KKumulaMDIWindow(KKumulaMainWindow):

    # the help menu is the latest inserted menu in the menubar
    # we override addHelpMenu() to insert the window menu BEFORE the help menu

    def addHelpMenu(self):
        self.actWindowCascade  = QAction(self, "actWindowCascade")
        self.actWindowTile     = QAction(self, "actWindowTile")
        self.actWindowClose    = QAction(self, "actWindowClose")
        self.actWindowCloseAll = QAction(self, "actWindowCloseAll")

        self.formatAction(self.actWindowCascade,  "Cascade",   "Cascade all windows.")
        self.formatAction(self.actWindowTile,     "Tile",      "Tile all windows.")
        self.formatAction(self.actWindowClose,    "Close",     "Close current window.")
        self.formatAction(self.actWindowCloseAll, "Close all", "Close all windows.")

        self.connect(self.actWindowCascade,  SIGNAL("activated()"), self.handleWindowCascade)
        self.connect(self.actWindowTile,     SIGNAL("activated()"), self.handleWindowTile)
        self.connect(self.actWindowClose,    SIGNAL("activated()"), self.handleWindowClose)
        self.connect(self.actWindowCloseAll, SIGNAL("activated()"), self.handleWindowCloseAll)

        self.menuWindow = self.makePopupMenu()
        self.connect(self.menuWindow, SIGNAL("aboutToShow()"), self.updateWindowMenu)

        self.MenuBar.insertItem(QString(""), self.menuWindow, 97)
        KKumulaMainWindow.addHelpMenu(self)



    def languageChange(self):
        KKumulaMainWindow.languageChange(self)
        self.MenuBar.findItem(97).setText("&Window")


    def updateWindowMenu(self):
        self.menuWindow.clear()
        self.actWindowCascade.addTo(self.menuWindow)
        self.actWindowTile.addTo(self.menuWindow)
        self.menuWindow.insertSeparator()
        self.actWindowClose.addTo(self.menuWindow)
        self.actWindowCloseAll.addTo(self.menuWindow)

        if (self.Workspace):
            if (len(self.Workspace.windowList()) > 0):
                self.actWindowCascade.setEnabled(1)
                self.actWindowTile.setEnabled(1)
                self.actWindowClose.setEnabled(1)
                self.actWindowCloseAll.setEnabled(1)
                self.menuWindow.insertSeparator()
                i = 0
                for win in self.Workspace.windowList():
                    winID = self.menuWindow.insertItem(win.caption(), self.handleWindowSelected)
                    self.menuWindow.setItemParameter(winID, i)
                    self.menuWindow.setItemChecked(winID, (win == self.Workspace.activeWindow()))
                    i = i + 1
            else:
                self.actWindowCascade.setEnabled(0)
                self.actWindowTile.setEnabled(0)
                self.actWindowClose.setEnabled(0)
                self.actWindowCloseAll.setEnabled(0)


    def handleWindowCascade(self):
        if (self.Workspace):
            self.Workspace.cascade()


    def handleWindowTile(self):
        if (self.Workspace):
            self.Workspace.tile()


    def handleWindowClose(self):
        if (self.Workspace):
            self.Workspace.activeWindow().close()


    def handleWindowCloseAll(self):
        if (self.Workspace):
            for win in self.Workspace.windowList():
                win.close()


    def handleWindowSelected(self, id):
        win = self.Workspace.windowList()[id]
        if (win):
            win.show()
            win.setFocus()


