# ============================================================
# KumulaMainWindow.py
# this is the common parent of all main windows.
# provides the help menu.
# ------------------------------------------------------------
# Copyright (C) 2004,2005 the Kumula Team.
# Licensed under the LGPL.
# ============================================================

import kumula
from qt import *
from KumulaAboutDialog import KKumulaAboutDialog

# ------------------------------------------------------------
# URLs for the Help Menu

linksTitles = [
    "Kumula Homepage",
    "Kumula @ KDE-Apps",
    "---",
    "Python Homepage",
    "Qt Homepage",
    "PyQt Homepage",
    "Eric Homepage",
    "---",
    "License (LGPL)",
    "No Software Patents"
]
linksURLs = [
    "http://www.kumula.org/",
    "http://www.kde-apps.org/content/show.php?content=11997",
    "",
    "http://www.python.org/",
    "http://www.trolltech.com/products/qt/",
    "http://www.riverbankcomputing.co.uk/pyqt/",
    "http://www.die-offenbachs.de/detlev/eric3.html",
    "",
    "http://www.opensource.org/licenses/lgpl-license.php",
    "http://www.nosoftwarepatents.com/"
]


# ------------------------------------------------------------


class KKumulaMainWindow(QMainWindow):

    def __init__(self):
        QMainWindow.__init__(self, None, "KumulaMainWindow", 0)
        self.initSettings()
        self.createMenuBar()
        self.createToolBar()
        self.createUI()
        self.addHelpMenu()
        self.languageChange()

        if (kumula.uiWinDim > 0):
            try:
                myWidth = kumula.ProgramSettings.readInt("window", "width", self.width())
                myHeight = kumula.ProgramSettings.readInt("window", "height", self.height())
                self.resize(QSize(myWidth,myHeight).expandedTo(self.minimumSizeHint()))
            except:
                # SettingsInterface not available
                pass



    def initSettings(self):
        # for reading settings and other things to be done
        # before creation of the user interface
        pass



    def createMenuBar(self):
        self.MenuBar = QMenuBar(self,"MenuBar")

    # Create menu entries for Internet Links

        self.menuProgInternet = self.makePopupMenu()
        for linkID in range(len(linksTitles)):
            if (linksTitles[linkID] == "---"):
                self.menuProgInternet.insertSeparator()
            else:
                menuEntryID = self.menuProgInternet.insertItem(QString(linksTitles[linkID]), self.handleProgInternetMenu)
                self.menuProgInternet.setItemParameter(menuEntryID, linkID)

    # Create program-related menu
        myAbout = QString("About ")
        myAbout.append(self.getProgramName())
        myAbout.append("...")
        myQuit = QString("Quit ")
        myQuit.append(self.getProgramName())

        self.actProgAnotherClient = KAction(self, "use_another_client", "Use another client...", self.handleProgAnotherClient, "Start this program with another client",    None,     1)
        self.actProgManual        = KAction(self, "manual",             "Manual",                self.handleHelpManual,        "Open the program's manual.",                "F1",     1)
        self.actProgAboutProgram  = KAction(self, "aboutprogram",       myAbout,                 self.handleHelpAboutProgram,  "Show information about this program.",      None,     1)
        self.actProgAboutKumula   = KAction(self, "aboutkumula",        "About Kumula Suite...", self.handleHelpAboutKumula,   "Show information about the program suite.", None,     1)
        self.actProgConfigure     = KAction(self, "configure",          "Run Configurator",      self.handleProgConfigure,     "Start the configuration program.",          None,     1)
        self.actProgQuit          = KAction(self, "quit",               myQuit,                  self.handleProgQuit,          "Quit this program.",                        "Alt+F4", 1)

#        self.actProgAnotherClient.menuAction.setIconSet(QIconSet(kumula.getMenuBarIcon("use_another_client",1)))
#        self.actProgManual.setIconSet(QIconSet(kumula.getMenuBarIcon("manual",1)))
#        self.actProgQuit.setIconSet(QIconSet(kumula.getMenuBarIcon("quit",1)))

        self.menuProg = self.makePopupMenu()
        if (kumula.ClientNeeded == 1):
            self.actProgAnotherClient.addToMenuBar(self.menuProg)
            self.menuProg.insertSeparator()
        self.actProgManual.addToMenuBar(self.menuProg)
        self.menuProg.insertSeparator()
        self.actProgAboutProgram.addToMenuBar(self.menuProg)
        self.actProgAboutKumula.addToMenuBar(self.menuProg)
        self.menuProg.insertSeparator()
        self.menuProg.insertItem(QString("Internet Links"), self.menuProgInternet, -2)
        self.menuProg.insertSeparator()
        if (kumula.ProgramName != "configurator"):
            self.actProgConfigure.addToMenuBar(self.menuProg)
            self.menuProg.insertSeparator()
        self.actProgQuit.addToMenuBar(self.menuProg)

        if (kumula.uiTextProgMenu == 0):
            self.MenuBar.insertItem(kumula.getMenuBarIcon("kumula",1), self.menuProg, -1)
        else:
            self.MenuBar.insertItem(QString("[K]"), self.menuProg, -1)


    def createToolBar(self):
        self.ToolBar = QToolBar(QString("Toolbar"), self, Qt.DockTop)


# ------------------------------------------------------------

# addHelpMenu is obsolete since kumula-2005.11,
# because the Help actions are now part of the
# program menu (on the left side, with icon).

# We let it here for a while.
# Could be useful in future to align a menu on the right.
# That's the reason why applications still should call it!

    def addHelpMenu(self):
        pass
#        self.MenuBar.insertItem(QString("Kumula"), self.menuProg, 99)


# ------------------------------------------------------------

    def makePopupMenu(self, aTitle=None):
        aPM = QPopupMenu(self, aTitle)
        if (kumula.uiTearOff > 0):
            aPM.insertTearOffHandle()
        return aPM


    def createUI(self):
        self.setCaption(self.getProgramName())
        self.setIcon(kumula.getProgramIcon())


    def getProgramName(self):
        return "A Kumula Application"
        
    def getProgramVersion(self):
        return "0.1"

    def getProgramCopyright(self):
        return "Copyright (C) 2004-2006 the Kumula Team"


    def formatAction(self, action, title, explanation=None, accel=None):
        action.setMenuText(title)
        if (explanation):
            action.setText(explanation)
        if (accel):
            action.setAccel(QKeySequence(accel))
        # set "What's this?"
        myWT = QString("<b>")
        myWT.append(title)
        myWT.append("</b>")
        if (explanation):
            myWT.append("<br><br>")
            myWT.append(explanation)
        action.setWhatsThis(myWT)


    def languageChange(self):
        pass


    def setCaption(self, caption):
        c = QString(caption)
        if (c.find("KUMULA") < 0):
            c.append(" - KUMULA")
        QMainWindow.setCaption(self, c)



    def closeEvent(self, event):
        try:
            kumula.ProgramSettings.writeInt("window", "width", self.width())
            kumula.ProgramSettings.writeInt("window", "height", self.height())
        except:
            # SettingsInterface not available
            pass
        QMainWindow.closeEvent(self, event)


    def handleProgAnotherClient(self):
        kumula.runKumulaApp(kumula.ProgramName, {})

    def handleProgInternetMenu(self, linkID):
        kumula.runExternalApp(kumula.AppBrowser, QString(linksURLs[linkID]))

    def handleProgConfigure(self):
        kumula.runKumulaApp("configurator")

    def handleProgQuit(self):
        kumula.Program.closeAllWindows()

    def handleHelpManual(self):
#        urlManual = QString(kumula.ProgramPath)
#        urlManual.append("/doc/")
#        urlManual.append(kumula.ProgramName)
#        urlManual.append("/index.html")
        urlManual = QString("http://www.kumula.org/docs/?title=")
        urlManual.append(self.getProgramName())
        kumula.runExternalApp(kumula.AppBrowser, urlManual)


    def handleHelpOnlineHomepage(self):
        global urlHomepage
        kumula.runExternalApp(kumula.AppBrowser, urlHomepage)


    def handleHelpOnlineBoard(self):
        global urlForums
        kumula.runExternalApp(kumula.AppBrowser, urlForums)


    def handleHelpOnlineReport(self):
        global urlBugzilla
        kumula.runExternalApp(kumula.AppBrowser, urlBugzilla)


    def handleHelpAboutProgram(self):
        myTitle = QString("About ")
        myTitle.append(self.getProgramName())

        myContent = QString("<h3>")
        myContent.append(self.getProgramName())
        myContent.append("</h3><p>Version ")
        myContent.append(self.getProgramVersion())
        myContent.append("</p><p>")
        myContent.append(self.getProgramCopyright())
        myContent.append("</p><p>This program belongs to the<br>Kumula Business Suite. Choose<br><i>Help/Internet Links/Kumula Homepage</i><br>for more details.</p>")

        QMessageBox.about(self, myTitle, myContent)


    def handleHelpAboutKumula(self):
        myDlg = KKumulaAboutDialog(kumula.MainWin)
        myDlg.exec_loop()





class KAction(object):

    def __init__(self, parent, name, title, method, explanation=None, accel=None, isProgAction=0):
        self.parent = parent
        self.name = name
        self.title = title
        self.method = method
        self.methodActive = 1
        self.explanation = explanation
        self.accel = accel
        self.isProgAction = isProgAction
        self.isTogglable = 0
        self.menuAction = QAction(self.parent, self.name)
        self.toolButton = None
        self.parent.connect(self.menuAction, SIGNAL("activated()"), self.handleMenuMethod)
        if not (self.method):
            self.menuAction.setEnabled(0)



    def addToMenuBar(self, aMenu):
        self.parent.formatAction(self.menuAction, self.title, self.explanation, self.accel)
        # if an icon named "m_<name>" exists, it will be used!
        if (self.isProgAction > 0):
            self.menuAction.setIconSet(QIconSet(kumula.getMenuBarIcon(self.name,1)))
        else:
            self.menuAction.setIconSet(QIconSet(kumula.getMenuBarIcon(self.name)))
        self.menuAction.addTo(aMenu)


    def addToToolBar(self, aToolbar, shortTitle=None):
        if self.toolButton:
            return
        self.toolButton = QToolButton(aToolbar)
        self.toolButton.setEnabled(self.menuAction.isEnabled())
        self.toolButton.setToggleButton(self.menuAction.isToggleAction())
        self.toolButton.setOn(self.menuAction.isOn())
        self.toolButton.setIconSet(QIconSet(kumula.getToolBarIcon(self.name)))
        if (shortTitle):
            self.toolButton.setTextLabel(shortTitle)
        else:
            self.toolButton.setTextLabel(self.title)

        if (kumula.uiToolButtons > 0):
            self.toolButton.setUsesTextLabel(1)
            if (kumula.uiToolButtons == 1):
                self.toolButton.setTextPosition(QToolButton.BelowIcon)
            elif (kumula.uiToolButtons == 2):
                self.toolButton.setTextPosition(QToolButton.BesideIcon)

        self.parent.connect(self.toolButton, SIGNAL("clicked()"), self.handleToolMethod)
        if not (self.method):
            self.toolButton.setEnabled(0)


    def setEnabled(self, aState):
        # a not-connected action can't be active!!!
        if (self.method):
            self.isEnabled = aState
        else:
            self.isEnabled = 0
        self.menuAction.setEnabled(self.isEnabled)
        if (self.toolButton):
            self.toolButton.setEnabled(self.menuAction.isEnabled())


    def setToggleAction(self, isTogglable):
        self.isTogglable = isTogglable
        self.menuAction.setToggleAction(self.isTogglable)
        if (self.toolButton):
            self.toolButton.setToggleButton(self.isTogglable)


    def setOn(self, aState, disableSignal=0):
        if (disableSignal > 0):
            self.methodActive = 0
        self.menuAction.setOn(aState)
        if (self.toolButton):
            self.toolButton.setOn(aState)
        self.methodActive = 1



    def isOn(self):
        return self.menuAction.isOn()


    def handleMenuMethod(self):
        # sync the toolbutton and the menuaction states
        if (self.toolButton) and (self.isTogglable > 0):
            self.setOn(self.menuAction.isOn(), 1)
        if (self.method) and (self.methodActive > 0):
            self.method()

    def handleToolMethod(self):
        # sync the toolbutton and the menuaction states
        if (self.toolButton) and (self.isTogglable > 0):
            self.setOn(self.toolButton.isOn(), 1)
        if (self.method) and (self.methodActive > 0):
            self.method()




class KProgramMenu(QCustomMenuItem):

    def __init__(self, f):
        self.string = QString("Kumula")
        self.font = f
        self.font.setBold(1)

    def paint(self, p, cg, act, enabled, x, y, w, h):
        p.setFont(self.font)
        p.drawText(x, y, w, h, AlignAuto or AlignVCenter or ShowPrefix or DontClip, self.string)

    def sizeHint(self):
        return QFontMetrics(self.font).size(AlignAuto or AlignVCenter or ShowPrefix or DontClip, self.string)

