# ============================================================
# SettingsManager.py
# manages user settings (global or program-specific) in the DB
# ------------------------------------------------------------
# Copyright (C) 2004 the Kumula Team.
# Licensed under the LGPL.
# ============================================================

from qt import QString, Qt
from qtsql import QSqlDatabase

# ------------------------------------------------------------


class KSettingsInterface(object):

    def __init__(self, aDB, aProgName, aUserName):
        self.DB  = aDB
        self.UserName    = aUserName
        self.ProgramName = aProgName



    def readString(self, aSection, aOption, aDefault=''):
        myValue   = QString(aDefault)
        quSetting = self.DB.doSelect("settings", ("value",), "", {"program":self.ProgramName, "section":aSection, "name":aOption, "user":self.UserName}, 1)
        while quSetting.next():
            myValue = quSetting.value(0).toString()

        # if myValue is equal to aDefault, delete this line from database
        # (helps to reduze database size a little bit)
        if (myValue == aDefault):
            self.DB.doDelete("settings", {"program":self.ProgramName, "section":aSection, "name":aOption, "user":self.UserName, "value":myValue}, 1)

        return myValue



    def readInt(self, aSection, aOption, aDefault=0):
        myDefault = QString("%i" % aDefault)
        myValue = self.readString(aSection, aOption, myDefault)
        myInt   = myValue.toInt()
        return myInt[0]
    


    def writeString(self, aSection, aOption, aValue):
        # first check if setting is already there...
        quSetting = self.DB.doSelect("settings", ("value",), "", {"program":self.ProgramName, "section":aSection, "name":aOption, "user":self.UserName}, 1)
        if quSetting.next():
            # ...then we need an update statement...
            self.DB.doUpdate("settings", {"value":aValue}, {"program":self.ProgramName, "section":aSection, "name":aOption, "user":self.UserName}, 1)
        else:
            # ...else we use an insert statement!
            self.DB.doInsert("settings", {"program":self.ProgramName, "section":aSection, "name":aOption, "value":aValue, "user":self.UserName})



    def writeInt(self, aSection, aOption, aValue):
        self.writeString(aSection, aOption, "%i" % aValue)


