# ============================================================
# SideBar.py
# new sidepanel for some main windows.
# ------------------------------------------------------------
# Copyright (C) 2004 the Kumula Team.
# Licensed under the LGPL.
# ============================================================

from qt import Qt,QColor,QFrame,QLabel,QPixmap,QSizePolicy,QSpacerItem,QVBoxLayout,PYSIGNAL
import kumula

# ------------------------------------------------------------

sbTextColor   = QColor(255, 255, 255)    # white
sbBarColor    = QColor(127, 127, 127)    # dark grey
sbActiveColor = QColor(170, 170, 170)    # lighter grey
#sbBarColor    = QColor(159, 159, 127)    # dark grey
#sbActiveColor = QColor(191, 191, 159)    # lighter grey

# ------------------------------------------------------------


class KSideBarButton(QFrame):

    def __init__(self, parent, name):
        QFrame.__init__(self, parent, name)
        self.setPaletteForegroundColor(sbTextColor)
        self.setFrameShadow(QFrame.Sunken)
        self.myLayout = QVBoxLayout(self, 2, 2)
        self.myPixmap = QLabel(self, "myPixmap")
        self.myPixmap.setAlignment(Qt.AlignHCenter)
        self.myLayout.addWidget(self.myPixmap)
        self.myLabel = QLabel(self, "myLabel")
        self.myLabel.setAlignment(Qt.AlignHCenter)
        self.myLayout.addWidget(self.myLabel)
        self.deactivate()
        self.enabled = 1

    def setPixmap(self, pixmap):
        if (pixmap):
            self.myEnabledPixmap  = QPixmap(pixmap)
            tempImage = self.myEnabledPixmap.convertToImage()
            self.myDisabledPixmap = QPixmap(self.myEnabledPixmap)
            self.myDisabledPixmap.convertFromImage(tempImage, QPixmap.Mono)
            self.myPixmap.setPixmap(self.myEnabledPixmap)
        else:
            self.myPixmap.setPixmap(QPixmap(""))


    def setLabel(self, label):
        self.myLabel.setText(label)

    def label(self):
        return self.myLabel


    def setTag(self, tag):
        self.tag = tag

    def setEnabled(self, isEnabled):
        self.enabled = isEnabled
        if (isEnabled == 0):
            self.myPixmap.setPixmap(self.myDisabledPixmap)
        else:
            self.myPixmap.setPixmap(self.myEnabledPixmap)

    def isEnabled(self):
        return self.enabled

    def activate(self):
        self.setPaletteBackgroundColor(sbActiveColor)
        self.setFrameShape(QFrame.Panel)

    def deactivate(self):
        self.setPaletteBackgroundColor(sbBarColor)
        self.setFrameShape(QFrame.NoFrame)

    def mousePressEvent(self, e):
        self.parent().buttonClicked(self.tag)



class KSideBar(QFrame):

    def __init__(self, parent, name):
        QFrame.__init__(self, parent, name)
        self.setPaletteForegroundColor(sbTextColor)
        self.setPaletteBackgroundColor(sbBarColor)
        self.setFrameShadow(QFrame.Sunken)
        self.setFrameShape(QFrame.Panel)
        self.myLayout = QVBoxLayout(self, 5, 5)
        self.Spacer  = ()
        self.Buttons = ()
        self.clear()
        self.adjustWidth()


    def clear(self):
        # this method was a challange for the brain!
        # it seems there are problems to remove the buttons
        # because of the QLayout and Pythons GC

        self.myLayout.deleteAllItems()
        for spacer in self.Spacer:
            spacer = None
        self.Spacer  = ()
        for btn in self.Buttons:
            btn.hide()
            btn = None
        self.Buttons = ()
        self.activeButton = None


    def adjustWidth(self):
        minWidth = 50
        for btn in self.Buttons:
            btnLabel = btn.label()
            btnWidth = btnLabel.fontMetrics().width(btnLabel.text())
            if (btnWidth > minWidth):
                minWidth = btnWidth
        minWidth = minWidth + 30
        self.setFixedWidth(minWidth)


    def insertButton(self, label, pixmap, tag):
        btn = KSideBarButton(self, "btn")
        btn.setPixmap(pixmap)
        btn.setLabel(label)
        btn.setTag(tag)
        btn.show()
        self.Buttons = self.Buttons + (btn,)
        self.myLayout.addWidget(btn)
        self.adjustWidth()


    def insertButtonAuto(self, label, pixmapName, tag):
        self.insertButton(label, kumula.getSideBarIcon(pixmapName), tag)


    def insertSpacer(self):
        spacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.Spacer = self.Spacer + (spacer,)
        self.myLayout.addItem(spacer)


    def selectButton(self, tag):
        hasSelected = 0
        for btn in self.Buttons:
            if (btn.tag == tag):
                if (btn.isEnabled() == 1):
                    btn.activate()
                    self.activeButton = btn
                    hasSelected = 1
        if (hasSelected == 1):
            for btn in self.Buttons:
                if (btn != self.activeButton):
                    btn.deactivate()
            self.selectionChanged(tag)


    def getButton(self, tag):
        for btn in self.Buttons:
            if (btn.tag == tag):
                return btn

    def enableAllButtons(self):
        for btn in self.Buttons:
            btn.setEnabled(1)


    def buttonClicked(self, tag):
        self.selectButton(tag)

    def selectionChanged(self, tag):
        self.emit(PYSIGNAL("selectionChanged()"), (tag,))

    def currentButton(self):
        return self.activeButton.tag

