

from widgetTmpl_unitconv import WidgetUnitConv
import kumula


UNITS = {
    "Length":{
        "Millimeter (mm)":      0.10,
        "Centimeter (cm)":      1.00,
        "Decimeter (dm)":      10.00,
        "Meter (m)":          100.00,
        "Kilometer (km)":  100000.00,
        "Inch (in)":            2.54,
        "Foot (ft)":           30.48,
        "Yard (yd)":           91.44,
        "Mile (stat.mi.)": 160934.00,
        "Nautical mile":   185200.00
        },
    "Mass":{
        "Milligramm (mg)":   0.001,
        "Gramm (g)":          1.00,
        "Kilogramm (kg)":  1000.00,
        "Tonne (t)":    1000000.00
    },
    "Time":{
        "Second (s)":     0.01667,
        "Minute (m)":     1.00,
        "Hour (h)":      60.00,
        "Day (d)":     1440.00,
        "Week (w)":   10080.00,
        "Year (y)":  525600.00
    },
    "SI units":{
        "nano (10 -9)":         0.000000001,
        "micro (10 -6)":        0.000001,
        "milli (10 -3)":        0.001,
        "centi (10 -2)":        0.01,
        "deci (10 -1)":         0.1,
        "none (10 0)":          1.0,
        "deca (10 1)":         10.0,
        "hecto (10 2)":       100.0,
        "kilo (10 3)":       1000.0,
        "mega (10 6)":    1000000.0,
        "giga (10 9)": 1000000000.0
    }
}


class CalcWidget(WidgetUnitConv):

    def __init__(self, parent):
        global UNITS

        WidgetUnitConv.__init__(self, parent)
        self.setIcon(kumula.getSideBarIcon("unitconv"))
        firstType = ""
        self.cbType.clear()
        for type in UNITS:
            self.cbType.insertItem(type)
            if (firstType == ""):
                firstType = type
        self.cbType.setCurrentText(firstType)
        self.updateUnits()


    def getDescription(self):
        return "Universal unit convertion"


    def updateUnits(self):
        global UNITS

        myType = self.cbType.currentText()
        myUnits = {}
        for type in UNITS:
            if (type == myType):
                myUnits = UNITS[type]
        self.cbFromUnit.clear()
        self.cbToUnit.clear()
        myStandardUnit = ""
        for unit in myUnits:
        #    print unit
            self.cbFromUnit.insertItem(unit)
            self.cbToUnit.insertItem(unit)
            if (myUnits[unit] == 1.0):
                myStandardUnit = unit
        self.cbFromUnit.setCurrentText(myStandardUnit)
        self.cbToUnit.setCurrentText(myStandardUnit)
        self.updateResult()


    def updateResult(self):
        global UNITS

        myType = self.cbType.currentText()
        myUnits = {}
        for type in UNITS:
            if (type == myType):
                myUnits = UNITS[type]

        for unit in myUnits:
            if (unit == self.cbFromUnit.currentText()):
                fromFactor = myUnits[unit]
            if (unit == self.cbToUnit.currentText()):
                toFactor = myUnits[unit]

        myValue = self.edFromValue.value()
        myResult = myValue * fromFactor / toFactor
        self.lblToValue.setText(QString("").setNum(myResult, 'f', 2))

