# ============================================================
# plugin.py ("INFO")
# information plugin for CLIENTS
# ------------------------------------------------------------
# Copyright (C) 2006 the Kumula Team.
# ============================================================

import kumula
from qt import *
from ClientsView import KClientsView

# ------------------------------------------------------------


class cvPlugin(KClientsView):

    # We have to save the temporary filenames (of contact photos) for later cleaning.
    # This is the only reason why this view has to reimplement it's constructor!

    def __init__(self, parent):
        KClientsView.__init__(self, parent)
        self.tempImages = ()

    # Because of Python's Garbage Collector, a destructor wouldn't be called in most cases.
    # So the cleanUp()-method was implemented in KClientsView...

    def cleanUp(self):
        for fileName in self.tempImages:
            os.unlink(fileName)


    # --------------------------------------------------------



    def getTitle(self):
        myTitle = QString("Info")
        return myTitle


    def handleAction(self, context, action, id):
        if (context == 'info'):
            if (action == 'logo'):
                self.editLogo()
            else:
                self.unknownCommand()
        else:
            self.unknownCommand()


    def getContent(self, cid):
        self.clientID = cid
        myLogoPath = QString("")
        content = QString('''
            <html>
            <body bgcolor="#FFFFFF" text="#000000" link="#0000FF" alink="#FF0000" vlink="#0000FF">
            <br>
            <table border="0" width="100%" cellspacing="5">
            <tr>
                <td align="center" valign="bottom" width="100%">
                ''')
        myLogo = self.getLogoFromDB(cid)
        if (myLogo):
            myLogo.prepend('<img src="')
            myLogo.append('">')
            myLink = self.createURL("info","logo","0")
            myLink.prepend('<a href="')
            myLink.append('">')
            myLogo.prepend(myLink)
            myLogo.append('</a>')
            content.append(myLogo)
        content.append('''
            </tr>
            </table>
            <table border="0" width="100%" cellspacing="5">
            <tr>
                <td align="right" valign="top" width="50%">''')
        quDescr = kumula.DB.doSelect("clients", ("description",), "id", {"id":cid})
        while quDescr.next():
            myDescription = quDescr.value(0).toString()
        if (myDescription):
            myDescription.replace("\n", "<br>")
            myDescription.prepend('<b>Description:</b><br>')
        else:
            myDescription = QString("<font color=#808080>No description.</font>")
        content.append(myDescription)

        content.append('''
                </td>
                <td align="left" valign="top" width="50%">''')

        optFound = 0
        quAttr = kumula.DB.doSelect("clients_attributes", ("id","title"), "title")
        while quAttr.next():
            myAttrID = quAttr.value(0).toInt()
            myAttrTitle = quAttr.value(1).toString()

            foundOptions = ()
            quOpts = kumula.DB.doSelect("clients_attroptions", ("id","title"), "title", {"attrid":"%i" % myAttrID})
            while quOpts.next():
                myOptID = quOpts.value(0).toInt()
                myOptTitle = quOpts.value(1).toString()
                quClientOpts = kumula.DB.doSelect("clients_2attr", ("optionid",), "optionid", {"clientid":cid,"optionid":"%i" % myOptID})
                while quClientOpts.next():
                    foundOptions = foundOptions + (myOptTitle,)
            if len(foundOptions) > 0:
                content.append("<b>")
                content.append(myAttrTitle)
                content.append(":</b><br><ul>")
                for optTitle in foundOptions:
                    optFound = optFound + 1
                    content.append("<li>")
                    content.append(optTitle)
                    content.append("</li>")
                content.append("</ul>")

        if (optFound == 0):
            content.append("<font color=#808080>No attributes.</font>")

        content.append('''</td>
            </tr>
            </table>
            </body>
            </html>
        ''')
        return content




    def moduleMenuItems(self):
        myItems = ("Set logo...",)
        return myItems


    def handleMenuItem(self, nr):
        if (nr == 0):
            self.editLogo()


    def editLogo(self):
        try:
            import Image
        except:
            result = QMessageBox.critical(kumula.MainWin, "Library missing",
                "To assign a logo to a client, you need to have the <b>Python Imaging Library (PIL)</b> installed.",
                "&OK")
            return
        try:
            sourceFileName = QFileDialog.getOpenFileName("~/", "All files (*.*)", kumula.MainWin, "LoadImgDlg", "Choose a picture as logo")
            if (sourceFileName):
                myPixmapFile = self.loadLogoFromFile(sourceFileName)
                self.saveLogoToDB(self.clientID, myPixmapFile)
        except:
            QMessageBox.warning(kumula.MainWin, "Error", "Can't open image file!", "&OK")
        self.reloadContent()




    def getLogoFromDB(self, cid):
        quData = kumula.DB.doSelect("clients_logos", ("logo",), "", {"clientid":cid})
        if quData.next():
            destFileName = tempfile.mktemp()
            myFile = QFile(destFileName)
            if (myFile.open(IO_WriteOnly)):
                myFile.writeBlock(quData.value(0).toByteArray())
                myFile.close()
            self.tempImages = self.tempImages + (destFileName,)
            return QString(destFileName)
        else:
            return None

    def loadLogoFromFile(self, aFileName):
        myWidth  = 200
        myHeight = 200
        try:
            import Image
            im = Image.open(aFileName.ascii())
            (imWidth,imHeight) = im.size
            # do resizing
            facW = imWidth  / float(myWidth)
            facH = imHeight / float(myHeight)
            if (facW > facH):
                myFactor = facW
            else:
                myFactor = facH
            destWidth  = int(imWidth  / myFactor)
            destHeight = int(imHeight / myFactor)
            im = im.resize((destWidth,destHeight), Image.ANTIALIAS)
        except:
            QMessageBox.critical(kumula.MainWin, "Can't create image",
                "Error during resizing/truncating of the image!", "&OK")
            return

        # do saving
        destFileName = tempfile.mktemp()
        try:
            im.save(destFileName, "JPEG")
        except:
            QMessageBox.critical(kumula.MainWin, "Can't create image",
                "The creation of the temporary image file has failed!", "&OK")
            return
        return destFileName


    def formatHexData(self, data):
        from binascii import hexlify
        myHex = "0x" + hexlify(data)
        return myHex


    def saveLogoToDB(self, cid, aFileName):
        # Delete old photo first
        # If the user didn't load a photo, the already stored one will be deleted, too.
        # It's not a bug, it's a feature! :-)
        kumula.DB.doDelete("clients_logos", {"clientid":cid}, 1)
        myFile = QFile(aFileName)
        if (myFile.open(IO_ReadOnly)):
            data = myFile.readBlock(65535)
            myStatement = QString("INSERT INTO clients_logos (clientid,logo) VALUES ('")
            myStatement.append(cid)
            myStatement.append("', ")
            myStatement.append(self.formatHexData(data))
            myStatement.append(" );")
            quInsert = kumula.DB.execStatement(myStatement)
            myFile.close()
        os.unlink(aFileName)

