# ============================================================
# ClientsView.py
# abstract template for viewer plugins
# ------------------------------------------------------------
# Copyright (C) 2004 the Kumula Team.
# Licensed under the LGPL.
# ============================================================

import kumula
from qt import *

# ------------------------------------------------------------


class KClientsView(QTextBrowser):


    #############################################################################
    # these methods have to be overridden!
    #############################################################################

    def getTitle(self):
        myTitle = QString("TEMPLATE")
        return myTitle

    
    def handleAction(self, context, action, id):
        self.unknownCommand()


    def getContent(self, cid):
        content = QString('''
            <html>
            <body bgcolor="#FFFFFF" text="#000000" link="#0000FF" alink="#FF0000" vlink="#0000FF">
            <br>This is an abstract template for viewer plugins!
            </body>
            </html>
        ''')
        return content


    def configureView(self):
        QMessageBox.information(kumula.MainWin, "Configure module", "This module doesn't have\nany settings.")


    def moduleMenuItems(self):
        # this abstract class doesn't have any menus.
        myItems = ()
        return myItems

    def handleMenuItem(self, nr):
        # this abstract class doesn't have any menus.
        pass


    def cleanUp(self):
        # this method should be overriden, if the view has to clean up something
        # at program exit (because Python's GC doesn't call __del__ correctly)
        pass


    def deleteClientData(self, aClientID):
        # this method is called whenever a client has to be deleted,
        # so the plugin can delete its own data assigned to this client.
        pass


    #############################################################################
    # for internal use!
    #############################################################################


    def createURL(self, context, action, id):   # 3x String
        url = QString(action)
        url.append(".")
        url.append(context)
        url.append(".")
        url.append(id)
        return url


    def setSource(self, url):
        params = QStringList.split('.', url)
        self.handleAction(params[1], params[0], params[2])


    def unknownCommand(self):
        QMessageBox.critical(kumula.MainWin, "Error", "Unknown command!")


    def loadContent(self, cid):
        self.cID = cid
        self.setText(self.getContent(cid))


    def reloadContent(self):
        self.loadContent(self.cID)


    def clearContent(self):
        self.setText('')

