# ============================================================
# ClientsWindow.py
# main window for the client management program.
# ------------------------------------------------------------
# Copyright (C) 2004-2005 the Kumula Team.
# ============================================================

import os
import kumula
from qt import *
from KumulaMainWindow import KKumulaMainWindow,KAction
from SideBar import KSideBar
from ModuleManager import KModuleManager
from ClientsView import KClientsView
from ClientDialog import KClientDialog
from SpecialItems import KIconViewItemWithID,KColoredListViewItem
from AttributesDialog import KAttributesDialog

# ------------------------------------------------------------


class KClientsWindow(KKumulaMainWindow):

    def getProgramName(self):
        return "Clients"

    def getProgramVersion(self):
        return "1.0"

    def getProgramCopyright(self):
        return "Copyright (C) 2006 Juergen Gerner."


    def initSettings(self):
        KKumulaMainWindow.initSettings(self)
        # initialize Settings
        self.colClient        = kumula.GlobalSettings.readString("colors", "Client", "#000000")
        self.colLostClient    = kumula.GlobalSettings.readString("colors", "LostClient", "#808080")
        self.colPotential     = kumula.GlobalSettings.readString("colors", "Potential", "#800000")
        self.colLostPotential = kumula.GlobalSettings.readString("colors", "LostPotential", "#FF8080")

        self.listWidth        = kumula.ProgramSettings.readInt("program", "listWidth",  200)
        self.lastClient       = kumula.ProgramSettings.readString("program", "lastClient", "")
        self.lastModule       = kumula.ProgramSettings.readString("program", "lastModule", "1info")

        self.viewRelatedClients   = kumula.GlobalSettings.readInt("clientsView", "relatedClients", 0)
        self.viewPotentialClients = kumula.GlobalSettings.readInt("clientsView", "potentialClients", 1)
        self.viewLostClients      = kumula.GlobalSettings.readInt("clientsView", "lostClients", 0)
        self.lastAttribute        = kumula.GlobalSettings.readInt("clientsView", "lastAttribute",  -1)
        self.lastAttrOption       = kumula.GlobalSettings.readInt("clientsView", "lastAttrOption",  -1)

        self.curAttribute = self.lastAttribute
        self.curClient = QString("")




    def createUI(self):
        KKumulaMainWindow.createUI(self)
        self.statusBar()
        # initalize user interface

        self.setCentralWidget(QWidget(self, "qt_central_widget"))
        myLayout = QGridLayout(self.centralWidget(), 0, 0, 0, 0, "myLayout")

        self.groupList = KSideBar(self.centralWidget(), "groupList")
        myLayout.addWidget(self.groupList, 0, 0)

        self.splitter = QSplitter(self.centralWidget(), "splitter")
        self.splitter.setOrientation(Qt.Horizontal)

        self.clientList = QListView(self.splitter, "clientList")
        self.clientList.addColumn("ID")
        self.clientList.addColumn("Name")
        self.clientList.setAllColumnsShowFocus(1)
        self.clientList.setResizeMode(QListView.LastColumn)

        # right side:
        self.clientInfoPanel = QFrame(self.splitter, "clientInfoPanel")
        panelLayout = QVBoxLayout(self.clientInfoPanel,0,0,"panelLayout")

        # right side: the frame for the client name and the "start" button
        self.clientHeader = QFrame(self.clientInfoPanel, "clientHeader")
        self.clientHeader.setPaletteBackgroundColor(QColor(127,127,127))
        self.clientHeader.setFrameShape(QFrame.StyledPanel)
        self.clientHeader.setFrameShadow(QFrame.Sunken)
        headerLayout = QHBoxLayout(self.clientHeader,6,6,"headerLayout")

        # right side: the QLabel used for the client's name
        self.clientLabel = QLabel(self.clientHeader, "clientLabel")
        self.clientLabel.setPaletteForegroundColor(QColor(255,255,255))
        clientLabel_font = QFont(self.clientLabel.font())
        clientLabel_font.setPointSize(18)
        clientLabel_font.setBold(1)
        self.clientLabel.setFont(clientLabel_font)
        headerLayout.addWidget(self.clientLabel)

        headerSpacer = QSpacerItem(70,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        headerLayout.addItem(headerSpacer)

        # right side: the "start" button
        self.startMenu = QPopupMenu(self)
        if (kumula.isAppInstalled("documents")):
            self.startAppAction = QAction(self, "startAppAction")
            self.startAppAction.addTo(self.startMenu)
            self.startAppAction.setMenuText("...manage documents")
            self.connect(self.startAppAction, SIGNAL("activated()"), self.startAppDocuments)
        if (kumula.isAppInstalled("journal")):
            self.startAppAction = QAction(self, "startAppAction")
            self.startAppAction.addTo(self.startMenu)
            self.startAppAction.setMenuText("...do accounting")
            self.connect(self.startAppAction, SIGNAL("activated()"), self.startAppJournal)
        if (kumula.isAppInstalled("letters")):
            self.startAppAction = QAction(self, "startAppAction")
            self.startAppAction.addTo(self.startMenu)
            self.startAppAction.setMenuText("...write new letter")
            self.connect(self.startAppAction, SIGNAL("activated()"), self.startAppLetters)
        if (self.startMenu.count() == 0):
            self.startAppAction = QAction(self, "startAppAction")
            self.startAppAction.addTo(self.startMenu)
            self.startAppAction.setMenuText("no additional programs found.")
            self.startAppAction.setEnabled(0)

        self.startButton = QPushButton(self.clientHeader, "startButton")
        self.startButton.setText("For this client:")
        self.startButton.setIconSet(QIconSet(kumula.getIconByName("startmenu")))
        self.startButton.setPopup(self.startMenu)
        headerLayout.addWidget(self.startButton)

        panelLayout.addWidget(self.clientHeader)

        # the tabs for the viewing plugins
        self.browserTabs = KModuleManager(self.clientInfoPanel, "browserTabs")
        self.browserTabs.loadModules(self.modulesMenu)
        self.browserTabs.activateModule(self.lastModule)

        panelLayout.addWidget(self.browserTabs)
        myLayout.addWidget(self.splitter, 0, 1)
        self.splitter.setResizeMode(self.clientList, QSplitter.KeepSize)

        self.connect(self.groupList,  PYSIGNAL("selectionChanged()"), self.changedGroup)
        self.connect(self.clientList, SIGNAL("selectionChanged()"), self.updateClientViews)
        self.updateSideBar(self.lastAttrOption)

        self.resize(QSize(600,450).expandedTo(self.minimumSizeHint()))
        self.clientList.resize(QSize(self.listWidth,self.clientList.height()).expandedTo(self.clientList.minimumSizeHint()))
        self.clearWState(Qt.WState_Polished)



    def createMenuBar(self):
        KKumulaMainWindow.createMenuBar(self)
        # initialize Actions
        self.actClientNew                  = KAction(self, "client_new",                  "&New...",                    self.handleClientNew,                    "Create a new client.")
        self.actClientEdit                 = KAction(self, "client_edit",                 "&Edit...",                   self.handleClientEdit,                   "Edit the selected client.")
        self.actClientDelete               = KAction(self, "client_delete",               "&Delete...",                 self.handleClientDelete,                 "Delete the selected client.")
        self.actClientRelation             = KAction(self, "client_relation",             "Is &related to me",          self.handleClientRelation,               "(Un)Marks the selected client as related to you.")
        self.actClientHandleRelations      = KAction(self, "client_handlerelations",      "&Handle all relations...",   self.handleClientAllRelations,           "Set the relations of this client to other users.")
        self.actClientSetAsClient          = KAction(self, "client_setasclient",          "&Client",                    self.handleClientSetAsClient,            "Marks the selected client as 'normal' client.")
        self.actClientSetAsLostClient      = KAction(self, "client_setaslostclient",      "&Lost client",               self.handleClientSetAsLostClient,        "Marks the selected client as lost client.")
        self.actClientSetAsPotential       = KAction(self, "client_setaspotential",       "&Potential client",          self.handleClientSetAsPotential,         "Marks the selected client as potential client.")
        self.actClientSetAsLostPotential   = KAction(self, "client_setaslostpotential",   "Lost po&tential client",     self.handleClientSetAsLostPotential,     "Marks the selected client as lost potential client.")
        self.actAttributeEdit              = KAction(self, "attribute_edit",              "&Manage attributes...",      self.handleAttributeEdit,                "Manage the various attributes for classifying all clients.")
        self.actViewShowOnlyRelatedClients = KAction(self, "view_showonlyrelatedclients", "Show only &related clients", self.handleSettingsViewRelatedClients,   "Limit the list to clients related to you.")
        self.actViewShowPotentialClients   = KAction(self, "view_showpotentialclients",   "Show &potential clients",    self.handleSettingsViewPotentialClients, "Potential clients (also lost potential clients) can be hidden or visible.")
        self.actViewShowLostClients        = KAction(self, "view_showlostclients",        "Show &lost clients",         self.handleSettingsViewLostClients,      "Lost clients (also lost potential clients) can be hidden or visible.")
        self.actViewSetColorClient         = KAction(self, "view_colorclient",            "&Clients...",                self.handleViewSetColorClient,           "Set the color for 'normal' clients.")
        self.actViewSetColorLostClient     = KAction(self, "view_colorlostclient",        "&Lost clients...",           self.handleViewSetColorLostClient,       "Set the color for lost clients.")
        self.actViewSetColorPotential      = KAction(self, "view_colorpotential",         "&Potential clients...",      self.handleViewSetColorPotential,        "Set the color for potential clients.")
        self.actViewSetColorLostPotential  = KAction(self, "view_colorlostpotential",     "Lost po&tential clients...", self.handleViewSetColorLostPotential,    "Set the color for lost potential clients.")
        self.actViewRefresh                = KAction(self, "view_refresh",                "&Refresh",                   self.handleViewRefresh,                  "Refreshes the client list and the modules.")
        self.actModulePrint                = KAction(self, "print",                       "&Print...",                  self.handleModulePrint,                  "Print the content of the current module.")
        self.actModuleConfigure            = KAction(self, "configure",                   "&Configure...",              self.handleModuleConfigure,              "Configure the current module.")

        self.actClientRelation.setToggleAction(1)
        self.actViewShowOnlyRelatedClients.setToggleAction(1)
        self.actViewShowOnlyRelatedClients.setOn(self.viewRelatedClients)

        self.actViewShowPotentialClients.setToggleAction(1)
        self.actViewShowPotentialClients.setOn(self.viewPotentialClients)

        self.actViewShowLostClients.setToggleAction(1)
        self.actViewShowLostClients.setOn(self.viewLostClients)

        # create Menu Structure
        self.clientSetAsMenu = self.makePopupMenu()
        self.actClientSetAsClient.addToMenuBar(self.clientSetAsMenu)
        self.actClientSetAsLostClient.addToMenuBar(self.clientSetAsMenu)
        self.actClientSetAsPotential.addToMenuBar(self.clientSetAsMenu)
        self.actClientSetAsLostPotential.addToMenuBar(self.clientSetAsMenu)

        self.clientMenu = self.makePopupMenu()
        self.actClientNew.addToMenuBar(self.clientMenu)
        self.actClientEdit.addToMenuBar(self.clientMenu)
#        self.actClientDelete.addToMenuBar(self.clientMenu)
        self.clientMenu.insertSeparator()
        self.actClientRelation.addToMenuBar(self.clientMenu)
#        self.actClientHandleRelations.addToMenuBar(self.clientMenu)
        self.clientMenu.insertSeparator()
        self.clientMenu.insertItem(QString("&Set status"), self.clientSetAsMenu, 11)
        self.MenuBar.insertItem(QString("&Client"), self.clientMenu, 1)

        self.attributeMenu = self.makePopupMenu()
        self.MenuBar.insertItem(QString("&Attributes"), self.attributeMenu, 2)

        self.modulesMenu = self.makePopupMenu()
        self.actModulePrint.addToMenuBar(self.modulesMenu)
        self.actModuleConfigure.addToMenuBar(self.modulesMenu)
        self.modulesMenu.insertSeparator()
        self.MenuBar.insertItem(QString("&Module"), self.modulesMenu, 3)

        self.viewSetColorMenu = self.makePopupMenu()
        self.actViewSetColorClient.addToMenuBar(self.viewSetColorMenu)
        self.actViewSetColorLostClient.addToMenuBar(self.viewSetColorMenu)
        self.actViewSetColorPotential.addToMenuBar(self.viewSetColorMenu)
        self.actViewSetColorLostPotential.addToMenuBar(self.viewSetColorMenu)

        self.viewSideBarAttrMenu = self.makePopupMenu()

        self.viewMenu = self.makePopupMenu()
        self.actViewShowPotentialClients.addToMenuBar(self.viewMenu)
        self.actViewShowLostClients.addToMenuBar(self.viewMenu)
        self.viewMenu.insertSeparator()
        self.actViewShowOnlyRelatedClients.addToMenuBar(self.viewMenu)
        self.viewMenu.insertSeparator()
        self.viewMenu.insertItem(QString("&Set color"), self.viewSetColorMenu, 12)
        self.viewMenu.insertSeparator()
        self.viewMenu.insertItem(QString("SideBar shows"), self.viewSideBarAttrMenu, 13)
        self.viewMenu.insertSeparator()
        self.actViewRefresh.addToMenuBar(self.viewMenu)
        self.MenuBar.insertItem(QString("&View"), self.viewMenu, 4)

        self.updateAttributeMenu()



    def updateAttributeMenu(self):
        self.attributeMenu.clear()
        self.viewSideBarAttrMenu.clear()
        attrFound = 0
        quAttributes = kumula.DB.doSelect("clients_attributes", ("id","title"), "title")
        while quAttributes.next():
            attrFound = attrFound + 1
            myAttrID    = quAttributes.value(0).toInt()
            myAttrTitle = quAttributes.value(1).toString()
            myAttrMenu = self.makePopupMenu()
            self.attributeMenu.insertItem(myAttrTitle, myAttrMenu, myAttrID)
            myItemID = self.viewSideBarAttrMenu.insertItem(myAttrTitle, self.handleViewSideBarAttribute)
            self.viewSideBarAttrMenu.setItemParameter(myItemID, myAttrID)

            quOptions = kumula.DB.doSelect("clients_attroptions", ("id","title"), "title", {"attrid":"%i" % myAttrID})
            while quOptions.next():
                myOptID    = quOptions.value(0).toInt()
                myOptTitle = quOptions.value(1).toString()
                optID = myAttrMenu.insertItem(myOptTitle, self.handleAttributeSelected)
                myAttrMenu.setItemParameter(optID, myOptID)
                if (self.curClient == ""):
                    myAttrMenu.setItemEnabled(optID, 0)
                else:
            # LOAD ALL OPTIONS OF THIS CLIENT!!!
                    quOptSelected = kumula.DB.doSelect("clients_2attr", ("optionid",), "clientid", {"clientid":self.curClient, "optionid":"%i" % myOptID})
                    while quOptSelected.next():
                        myAttrMenu.setItemChecked(optID, 1)

        if (attrFound > 0):
            self.attributeMenu.insertSeparator()
        self.actAttributeEdit.addToMenuBar(self.attributeMenu)



    def createToolBar(self):
        KKumulaMainWindow.createToolBar(self)
        # initalize ToolBar
        self.actClientNew.addToToolBar(self.ToolBar, "New client")
        self.actClientEdit.addToToolBar(self.ToolBar, "Edit client")
#        self.actClientDelete.addToToolBar(self.ToolBar, "Delete client")
        self.ToolBar.addSeparator()
        self.actClientRelation.addToToolBar(self.ToolBar, "Is related")
        self.actViewShowOnlyRelatedClients.addToToolBar(self.ToolBar, "Show only related")
        self.actViewRefresh.addToToolBar(self.ToolBar, "Refresh")
        self.ToolBar.addSeparator()
        self.actModulePrint.addToToolBar(self.ToolBar, "Print")
        self.actModuleConfigure.addToToolBar(self.ToolBar, "Configure")


    def closeEvent(self, e):
        kumula.ProgramSettings.writeInt("program", "listWidth",  self.clientList.width())
        kumula.ProgramSettings.writeInt("program", "lastGroup",  self.groupList.currentButton())
        kumula.ProgramSettings.writeString("program", "lastClient", self.curClient)
        kumula.ProgramSettings.writeString("program", "lastModule",   self.browserTabs.activeModule())
        self.browserTabs.cleanUpModules()
        KKumulaMainWindow.closeEvent(self, e)



    def updateAllViews(self):
        self.browserTabs.updateModules(self.curClient)
        self.startMenu.setEnabled(1)

    def clearAllViews(self):
        self.browserTabs.clearModules()
        self.clientLabel.setText("")
        self.startMenu.setEnabled(0)


    def colorItem(self, item, status):
        if (status == 1):
            item.setColor(self.colLostClient)
        elif (status == 2):
            item.setColor(self.colPotential)
        elif (status == 3):
            item.setColor(self.colLostPotential)
        else:
            item.setColor(self.colClient)


    def updateSideBar(self, selOpt=None):
        if (not selOpt):
            selOpt = self.groupList.currentButton()  # <- could be "None", too!
        self.disconnect(self.groupList,  PYSIGNAL("selectionChanged()"), self.changedGroup)
        self.disconnect(self.clientList, SIGNAL("selectionChanged()"), self.updateClientViews)
        self.groupList.clear()
        self.clearAllViews()
        foundOpt = 0
        # Item for ALL clients
        self.groupList.insertButtonAuto("All clients", "folder_global", -1)
        quOptions = kumula.DB.doSelect("clients_attroptions", ("id","title","icon"), "title", {"attrid":"%i" % self.curAttribute})
        while quOptions.next():
            optID    = quOptions.value(0).toInt()
            optTitle = quOptions.value(1).toString()
            optIcon  = quOptions.value(2).toString()
            optIcon.prepend("attricons/")
            optIcon.append(".png")
            self.groupList.insertButton(optTitle, QPixmap(QString(os.path.join(kumula.getIconDir(), optIcon.ascii()))), optID)
            if (optID == selOpt):
                foundOpt = 1
        self.groupList.insertSpacer()

        self.connect(self.groupList,  PYSIGNAL("selectionChanged()"), self.changedGroup)
        self.connect(self.clientList, SIGNAL("selectionChanged()"), self.updateClientViews)
        # What if no clients are in this group?
        # -> ClientBrowser will not be updated!!
        if (foundOpt > 0):
            self.groupList.selectButton(selOpt)
        else:
            self.groupList.selectButton(-1)



    def changedGroup(self):
        # Workaround, because SideBar.selectionChanged would tell
        # self.updateClientList the GROUP id instead of the CLIENT id
        kumula.GlobalSettings.writeInt("clientsView", "lastAttrOption", self.groupList.currentButton())
        self.updateClientList()


    def updateClientList(self, selectedClient=''):
        # save ID of the selected client, and if possible, select it after updating the list
        if (selectedClient == ''):
            if self.clientList.currentItem():
                selectedClient = self.clientList.currentItem().text(0)
            else:
                # this is when the program starts
                selectedClient = self.lastClient
        self.clientList.clear()
        hasSelected = 0
        # fantastic: If the group with this name doesn't exist,
        # all clients from any group will be displayed...
        myGroupID = self.groupList.currentButton()
        if (myGroupID == -1):
            quClients = kumula.DB.doSelect("clients", ("id",), "id")
        else:
            quClients = kumula.DB.doSelect("clients_2attr", ("clientid",), "clientid", {"optionid":"%i" % myGroupID})
        while quClients.next():
            clID = quClients.value(0).toString()
            clStatus = kumula.ClientIntf.getClientStatus(clID)

        # by default, all clients are visible...
            visibleItem = 1
        # ...hide if client is lost (or lost+potential)
            if (self.viewLostClients == 0):
                if (clStatus == 1) or (clStatus == 3):
                    visibleItem = 0
        # ...hide if client is a potential (or lost+potential)
            if (self.viewPotentialClients == 0):
                if (clStatus == 2) or (clStatus == 3):
                    visibleItem = 0
        # ...hide if client is not related to the user
            if (self.viewRelatedClients == 1):
                if (not kumula.ClientIntf.isUserRelated(clID, kumula.UserName)):
                    visibleItem = 0
            if (visibleItem == 1):
                clName = kumula.ClientIntf.getClientName(clID)
                item   = KColoredListViewItem(self.clientList)
                item.setText(0, clID)
                item.setText(1, clName)
                self.colorItem(item, clStatus)
                if (clID == selectedClient):
                    self.clientList.setSelected(item, 1)
                    self.clientList.setCurrentItem(item)
                    hasSelected = 1
        self.actClientEdit.setEnabled(1)
        self.actClientDelete.setEnabled(1)
        self.actClientRelation.setEnabled(1)
        self.actModulePrint.setEnabled(1)
        self.actModuleConfigure.setEnabled(1)
        self.MenuBar.setItemEnabled(3, 1)
        self.MenuBar.setItemEnabled(11, 1)
        if (hasSelected == 0):
            # always select at least one client
            if self.clientList.firstChild():
                self.clientList.setCurrentItem(self.clientList.firstChild())
                self.clientList.setSelected(self.clientList.firstChild(), 1)
            else:
                # ...no client is in this group...
                self.actClientEdit.setEnabled(0)
                self.actClientDelete.setEnabled(0)
                self.actClientRelation.setEnabled(0)
                self.actClientRelation.setOn(0, 1)
                self.actModulePrint.setEnabled(0)
                self.actModuleConfigure.setEnabled(0)
                self.MenuBar.setItemEnabled(3, 0)
                self.MenuBar.setItemEnabled(11, 0)
        self.updateClientViews()


    def updateClientViews(self):
        self.curClient = QString("")
        if self.clientList.currentItem():
            self.curClient = self.clientList.currentItem().text(0)
        if (self.curClient == ""):
            self.clearAllViews()
        else:
            myLabel = QString(kumula.ClientIntf.getClientName(self.curClient))
            myLabel.append(" (")
            myLabel.append(self.curClient)
            myLabel.append(")")
            self.clientLabel.setText(myLabel)
            self.actClientRelation.setOn(kumula.ClientIntf.isUserRelated(self.curClient, kumula.UserName))
            self.updateAllViews()
        self.updateAttributeMenu()



    def startAppDocuments(self):
        kumula.runKumulaApp("documents", {"client":self.curClient})

    def startAppJournal(self):
        kumula.runKumulaApp("journal", {"client":self.curClient})

    def startAppLetters(self):
        kumula.runKumulaApp("letters", {"client":self.curClient})



    def __tr(self,s,c = None):
        return qApp.translate("MainWindow",s,c)



    def handleClientNew(self):
        dlg = KClientDialog()
        dlg.setCaption("Create new client")
        dlg.edID.setEnabled(1)
        dlg.edID.setText("")
        dlg.edName.setText("")
        dlg.edDescription.setText("")

        if dlg.exec_loop() == QDialog.Accepted:
            result = kumula.ClientIntf.newClient(dlg.edID.text(), dlg.edName.text())
            if (result >= 0):
                kumula.ClientIntf.setClientDescription(dlg.edID.text(), dlg.edDescription.text())
                # set the current attribute/option to the new client to make sure it appears on the list!
                selOpt = self.groupList.currentButton()
                if (selOpt > 0):
                    kumula.DB.doInsert("clients_2attr", {"clientid":dlg.edID.text(), "optionid":"%i" % selOpt})
            self.updateClientList(dlg.edID.text())


    def handleClientEdit(self):
        dlg = KClientDialog()
        dlg.setCaption("Edit this client")
        dlg.edID.setEnabled(0)
        dlg.btnSuggest.setEnabled(0)
        dlg.edID.setText(self.curClient)
        dlg.edName.setText(kumula.ClientIntf.getClientName(self.curClient))
        dlg.edDescription.setText(kumula.ClientIntf.getClientDescription(self.curClient))

        if dlg.exec_loop() == QDialog.Accepted:
            kumula.ClientIntf.setClientName(self.curClient, dlg.edName.text())
            kumula.ClientIntf.setClientDescription(self.curClient, dlg.edDescription.text())
            # We have to update the whole list here, because
            # the group assignments could have changed.
            self.updateClientList(self.curClient)


    def handleClientDelete(self):
        myQuestion = QString("Do you really want to<br>delete this client?<br><br><b>")
        myQuestion.append(kumula.ClientIntf.getClientName(self.curClient))
        myQuestion.append(" (")
        myQuestion.append(self.curClient)
        myQuestion.append(")</b><br><br>Warning! All attributes,<br>including contact persons,")
        myQuestion.append("<br>addresses and communication<br>data will also be deleted!")

        result = QMessageBox.warning(kumula.MainWin, "Question", myQuestion, "&Delete", "&Cancel")
        if result == 0:
            # it makes me a little bit nervous to see how easy this can be...
            # ...maybe we add an additional dialog ... or disable the whole functionality
            kumula.ClientIntf.deleteClient(self.curClient)
            # Don't just update the list anymore, to avoid unnecessary database access.
            # This also helps against this nervous flickering.
            # self.updateClientList()
            delItem = self.clientList.currentItem()
            nextItem = delItem.itemBelow()
            if (nextItem == None):
                nextItem = delItem.itemAbove()
            if (nextItem):
                self.clientList.setSelected(nextItem, 1)
            self.clientList.takeItem(delItem)


    def handleClientRelation(self):
        if (kumula.ClientIntf.isUserRelated(self.curClient, kumula.UserName)):
            kumula.ClientIntf.setUserRelation(self.curClient, kumula.UserName, 0)
        else:
            kumula.ClientIntf.setUserRelation(self.curClient, kumula.UserName, 1)
        self.actClientRelation.setOn(kumula.ClientIntf.isUserRelated(self.curClient, kumula.UserName))



    def handleClientAllRelations(self):
        pass


    def setStatusINTERNAL(self, status):
        kumula.ClientIntf.setClientStatus(self.curClient, status)
        item = self.clientList.currentItem()
        self.colorItem(item, status)

    def handleClientSetAsClient(self):
        self.setStatusINTERNAL(0)

    def handleClientSetAsLostClient(self):
        self.setStatusINTERNAL(1)

    def handleClientSetAsPotential(self):
        self.setStatusINTERNAL(2)

    def handleClientSetAsLostPotential(self):
        self.setStatusINTERNAL(3)



    def handleAttributeEdit(self):
        dlg = KAttributesDialog(self)
        dlg.exec_loop()
        self.updateAttributeMenu()
        self.updateSideBar()



    def handleAttributeSelected(self, optID):
        if (self.curClient == ""):
            return

        # is the attribute of this option exclusive or not?
        myExclusive = 0
        quOption = kumula.DB.doSelect("clients_attroptions", ("attrid",), "attrid", {"id":"%i" % optID})
        while quOption.next():
            myAttrID = quOption.value(0).toInt()
            quAttr = kumula.DB.doSelect("clients_attributes", ("exclusive",), "exclusive", {"id":"%i" % myAttrID})
            while quAttr.next():
                myExclusive = quAttr.value(0).toInt()

        if (myExclusive > 0):
        # if exclusive, delete all client's options of this attribute...
            quAttrOptions = kumula.DB.doSelect("clients_attroptions", ("id",), "id", {"attrid":"%i" % myAttrID})
            while quAttrOptions.next():
                myOptID = quAttrOptions.value(0).toInt()
                kumula.DB.doDelete("clients_2attr", {"clientid":self.curClient, "optionid":"%i" % myOptID})
        # ... and set this client's option
            kumula.DB.doInsert("clients_2attr", {"clientid":self.curClient, "optionid":"%i" % optID})
        else:
        # if not exclusive, TOGGLE this option!!!
            found = 0
            quClientOpt = kumula.DB.doSelect("clients_2attr", ("optionid",), "optionid", {"clientid":self.curClient,"optionid":"%i" % optID})
            while quClientOpt.next():
                found = 1
            if (found > 0):
                kumula.DB.doDelete("clients_2attr", {"clientid":self.curClient, "optionid":"%i" % optID})
            else:
                kumula.DB.doInsert("clients_2attr", {"clientid":self.curClient, "optionid":"%i" % optID})
        self.updateAttributeMenu()
        self.updateAllViews()



    def handleSettingsViewRelatedClients(self):
        self.viewRelatedClients = int(self.actViewShowOnlyRelatedClients.isOn())
        kumula.GlobalSettings.writeInt("clientsView", "relatedClients", self.viewRelatedClients)
        self.updateClientList()


    def handleSettingsViewPotentialClients(self):
        self.viewPotentialClients = int(self.actViewShowPotentialClients.isOn())
        kumula.GlobalSettings.writeInt("clientsView", "potentialClients", self.viewPotentialClients)
        self.updateClientList()


    def handleSettingsViewLostClients(self):
        self.viewLostClients = int(self.actViewShowLostClients.isOn())
        kumula.GlobalSettings.writeInt("clientsView", "lostClients", self.viewLostClients)
        self.updateClientList()


    def handleViewSetColorClient(self):
        myColor = QColorDialog.getColor(QColor(self.colClient), self)
        if myColor.isValid():
            self.colClient = myColor.name()
            kumula.GlobalSettings.writeString("colors", "Client", self.colClient)
            self.updateClientList()


    def handleViewSetColorLostClient(self):
        myColor = QColorDialog.getColor(QColor(self.colLostClient), self)
        if myColor.isValid():
            self.colLostClient = myColor.name()
            kumula.GlobalSettings.writeString("colors", "LostClient", self.colLostClient)
            self.updateClientList()


    def handleViewSetColorPotential(self):
        myColor = QColorDialog.getColor(QColor(self.colPotential), self)
        if myColor.isValid():
            self.colPotential = myColor.name()
            kumula.GlobalSettings.writeString("colors", "Potential", self.colPotential)
            self.updateClientList()


    def handleViewSetColorLostPotential(self):
        myColor = QColorDialog.getColor(QColor(self.colLostPotential), self)
        if myColor.isValid():
            self.colLostPotential = myColor.name()
            kumula.GlobalSettings.writeString("colors", "LostPotential", self.colLostPotential)
            self.updateClientList()


    def handleViewSideBarAttribute(self, attrID):
        self.curAttribute = attrID
        kumula.GlobalSettings.writeInt("clientsView", "lastAttribute",  attrID)
        self.updateSideBar(-1)


    def handleViewRefresh(self):
        self.updateClientList()


    def handleModulePrint(self):
        myViewer = self.browserTabs.currentModule()
        myPrinter = QPrinter(QPrinter.HighResolution)
        myPrinter.setFullPage(1)
        if myPrinter.setup():
            p = QPainter(myPrinter)
            if not p.isActive():
                # can't print. Raise error message.
                return
            metrics = QPaintDeviceMetrics(p.device())
            dpiy    = metrics.logicalDpiY()
            margin  = int((2/2.54)*dpiy)
            body    = QRect(margin, margin, metrics.width()-(2*margin), metrics.height()-(2*margin))
            richText = QSimpleRichText(myViewer.text(), QFont(), myViewer.context(), myViewer.styleSheet(), myViewer.mimeSourceFactory(), body.height())
            richText.setWidth(p, body.width())
            view = QRect(body)
            page = 1
            while (1 == 1):
                richText.draw(p, body.left(), body.top(), view, self.colorGroup())
                view.moveBy(0, body.height())
                p.translate(0, -body.height())
                p.drawText(view.right()-p.fontMetrics().width(str(page)), view.bottom()+p.fontMetrics().ascent()+5, str(page))
                if (view.top() >= richText.height()):
                    break
                myPrinter.newPage()
                page = page + 1
            p.end()



    def handleModuleConfigure(self):
        viewer = self.browserTabs.currentModule()
        viewer.configureView()




