# ============================================================
# ModuleManager.py
# QTabWidget with module managing qualities.
# ------------------------------------------------------------
# Copyright (C) 2004 the Kumula Team.
# ============================================================

import kumula
from qt import *

import sys
import os

# ------------------------------------------------------------


class KModuleManager(QTabWidget):

    def __init__(self, parent, name):
        QTabWidget.__init__(self, parent, name)
        self.setTabPosition(QTabWidget.Bottom)


    def loadModules(self, aModulesMenu):
        myViewerDir = os.path.join(os.path.dirname(sys.argv[0]), "..", "lib", "clients")
        # get list of all subdirectories of KUMULADIR/lib/clients
        tmpDirList = ()
        osDirList = os.listdir(myViewerDir)
        osDirList.sort()
        for Dir in osDirList:
            newDir = os.path.join(myViewerDir, Dir)
            if os.path.isdir(newDir):
                tmpDirList = tmpDirList + (Dir,)
                sys.path.insert(0, newDir)

        # init "translation" dictionaries for module-specific menus
        self.moduleByID       = {}
        self.moduleName2ID    = {}
        self.moduleByMenu     = {}
        self.moduleHandleID   = {}
        TabID = 0

        # in each subdirectory, load the file "plugin.py"...
        for fi in tmpDirList:
            myFilePath = os.path.join(myViewerDir, fi, "plugin.py")
            myPlugin = open(myFilePath, "r")
            # ...compile it...
            exec myPlugin in locals()
            myPlugin.close()
            # ...and add it to our tab bar
            self.tab = QWidget(self,"tab")
            tabLayout = QGridLayout(self.tab,1,1,0,6,"tabLayout")
            myView = cvPlugin(self.tab)
            myView.setFrameShape(QTextBrowser.NoFrame)
            tabLayout.addWidget(myView,0,0)

            myTitle = myView.getTitle()
            self.insertTab(self.tab, myTitle)
            self.moduleName2ID[fi] = TabID
            self.moduleByID[TabID] = myView
            TabID = TabID + 1

            # define Module menu -------------------------
            mySubMenu = QPopupMenu(self)
#            actID = mySubMenu.insertItem("Show", self.handleModuleShow)
#            self.moduleByMenu[actID] = myView
#            self.moduleHandleID[actID] = -1

#            actID = mySubMenu.insertItem("Configure...", self.handleModuleConfigure)
#            self.moduleByMenu[actID] = myView
#            self.moduleHandleID[actID] = -2

            menuID = 0
            for title in myView.moduleMenuItems():
#                if (menuID == 0):
#                    mySubMenu.insertSeparator()
                if (title == "---"):
                    mySubMenu.insertSeparator()
                else:
                    actID = mySubMenu.insertItem(title, self.handleModuleMenu)
                    self.moduleByMenu[actID] = myView
                    self.moduleHandleID[actID] = menuID
                    menuID = menuID + 1

            aModulesMenu.insertItem(myTitle, mySubMenu, menuID)
            menuID = menuID + 1


    def activateModule(self, aName):
        for myName in self.moduleName2ID:
            if (aName == myName):
                myIndex = self.moduleName2ID[myName]
                self.setCurrentPage(myIndex)


    def activeModule(self):
        curName = ""
        for myName in self.moduleName2ID:
            if (self.moduleName2ID[myName] == self.currentPageIndex()):
                curName = myName
        return curName


    def clearModules(self):
        for view in self.moduleByID:
            self.moduleByID[view].clearContent()

    def updateModules(self, aClientID):
        for view in self.moduleByID:
            self.moduleByID[view].loadContent(aClientID)

    def currentModule(self):
        i = self.currentPageIndex()
        return self.moduleByID[i]


    def cleanUpModules(self):
        for view in self.moduleByID:
            self.moduleByID[view].cleanUp()


# -----------------------------------------
# ROUTINES FOR HANDLING MODULE MENU ENTRIES
# -----------------------------------------


    def handleModuleShow(self, modID):
        myMod = self.moduleByMenu[modID]
        for view in self.moduleByID:
            if (self.moduleByID[view] == myMod):
                self.setCurrentPage(view)


    def handleModuleConfigure(self, modID):
        myMod = self.moduleByMenu[modID]
        myMod.configureView()


    def handleModuleMenu(self, modID):
        myMod  = self.moduleByMenu[modID]
        myMenu = self.moduleHandleID[modID]
        myMod.handleMenuItem(myMenu)


