# ============================================================
# AddressDialog.py
# dialog for address data (comment, zone, address, ...)
# ------------------------------------------------------------
# Copyright (C) 2004 the Kumula Team.
# ============================================================

import kumula
from qt import *
from SpecialItems import KListViewItemWithID

# ------------------------------------------------------------


class KAddressDialog(QDialog):


    def __init__(self,parent = None,name = None,modal = 0,fl = 0):
        QDialog.__init__(self,parent,name,modal,fl)
        if not name:
            self.setName("KAddressDlg")

        KAddressDlgLayout = QGridLayout(self,1,1,11,6,"KAddressDlgLayout")

        # Comment for this address
        self.cbComment = QComboBox(0,self,"cbComment")
        self.cbComment.setEditable(1)
        KAddressDlgLayout.addWidget(self.cbComment,0,1)

        self.lblComment = QLabel(self,"lblComment")
        self.lblComment.setAlignment(QLabel.AlignVCenter | QLabel.AlignRight)
        KAddressDlgLayout.addWidget(self.lblComment,0,0)

        # GroupBox for the complete address
        self.gbAddress = QGroupBox(self,"gbAddress")
        self.gbAddress.setColumnLayout(0,Qt.Vertical)
        self.gbAddress.layout().setSpacing(6)
        self.gbAddress.layout().setMargin(11)
        gbAddressLayout = QGridLayout(self.gbAddress.layout())
        gbAddressLayout.setAlignment(Qt.AlignTop)

        self.lblTitle = QLabel(self.gbAddress,"lblTitle")
        gbAddressLayout.addWidget(self.lblTitle,0,0)
        self.edTitle = QLineEdit(self.gbAddress,"edTitle")
        gbAddressLayout.addMultiCellWidget(self.edTitle,0,0,1,2)

        self.edReceiver = QLineEdit(self.gbAddress,"edReceiver")
        gbAddressLayout.addMultiCellWidget(self.edReceiver,1,1,1,2)
        self.lblReceiver = QLabel(self.gbAddress,"lblReceiver")
        gbAddressLayout.addWidget(self.lblReceiver,1,0)

        self.edRecComment = QLineEdit(self.gbAddress,"edRecComment")
        gbAddressLayout.addMultiCellWidget(self.edRecComment,2,2,1,2)
        self.lblRecComment = QLabel(self.gbAddress,"lblRecComment")
        gbAddressLayout.addWidget(self.lblRecComment,2,0)

        self.edStreet = QLineEdit(self.gbAddress,"edStreet")
        gbAddressLayout.addMultiCellWidget(self.edStreet,3,3,1,2)
        self.lblStreet = QLabel(self.gbAddress,"lblStreet")
        gbAddressLayout.addWidget(self.lblStreet,3,0)

        self.lblZip = QLabel(self.gbAddress,"lblZip")
        gbAddressLayout.addWidget(self.lblZip,4,0)

        self.edZip = QLineEdit(self.gbAddress,"edZip")
        self.edZip.setMaxLength(7)
        self.edZip.setFixedWidth(60)
        gbAddressLayout.addWidget(self.edZip,4,1)

        self.cbCity = QComboBox(0,self.gbAddress,"cbCity")
        self.cbCity.setEditable(1)
        self.cbCity.setAutoCompletion(1)
        gbAddressLayout.addWidget(self.cbCity,4,2)

        self.edDistrict = QLineEdit(self.gbAddress,"edDistrict")
        gbAddressLayout.addMultiCellWidget(self.edDistrict,5,5,1,2)
        self.lblDistrict = QLabel(self.gbAddress,"lblDistrict")
        gbAddressLayout.addWidget(self.lblDistrict,5,0)

        KAddressDlgLayout.addWidget(self.gbAddress,2,1)

        # ListView for country / zone
        self.lvCountryZone = QListView(self,"lvCountryZone")
        self.lvCountryZone.addColumn(self.__tr("Country / zone:"))
        self.lvCountryZone.setRootIsDecorated(1)
        self.lvCountryZone.setResizeMode(QListView.LastColumn)
        self.lvCountryZone.setFixedWidth(230)
        KAddressDlgLayout.addWidget(self.lvCountryZone,2,0)

        self.line3 = QFrame(self,"line3")
        self.line3.setFrameShape(QFrame.HLine)
        self.line3.setFrameShadow(QFrame.Sunken)
        self.line3.setFrameShape(QFrame.HLine)
        KAddressDlgLayout.addMultiCellWidget(self.line3,1,1,0,1)

        self.line2_2 = QFrame(self,"line2_2")
        self.line2_2.setFrameShape(QFrame.HLine)
        self.line2_2.setFrameShadow(QFrame.Sunken)
        self.line2_2.setFrameShape(QFrame.HLine)
        KAddressDlgLayout.addMultiCellWidget(self.line2_2,3,3,0,1)

        layout2 = QHBoxLayout(None,0,6,"layout2")
        spacer1 = QSpacerItem(40,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        layout2.addItem(spacer1)

        self.btnOK = QPushButton(self,"btnOK")
        layout2.addWidget(self.btnOK)
        self.btnCancel = QPushButton(self,"btnCancel")
        layout2.addWidget(self.btnCancel)

        KAddressDlgLayout.addMultiCellLayout(layout2,4,4,0,1)

        self.languageChange()

        self.resize(QSize(600,300).expandedTo(self.minimumSizeHint()))
        self.clearWState(Qt.WState_Polished)

        self.connect(self.btnOK,SIGNAL("clicked()"),self,SLOT("accept()"))
        self.connect(self.btnCancel,SIGNAL("clicked()"),self,SLOT("reject()"))
        self.connect(self.edZip,SIGNAL("textChanged(const QString&)"),self.updateCity)

        self.setTabOrder(self.cbComment,self.lvCountryZone)
        self.setTabOrder(self.lvCountryZone,self.edTitle)
        self.setTabOrder(self.edTitle,self.edReceiver)
        self.setTabOrder(self.edReceiver,self.edRecComment)
        self.setTabOrder(self.edRecComment,self.edStreet)
        self.setTabOrder(self.edStreet,self.edZip)
        self.setTabOrder(self.edZip,self.cbCity)
        self.setTabOrder(self.cbCity,self.edDistrict)
        self.setTabOrder(self.edDistrict,self.btnOK)
        self.setTabOrder(self.btnOK,self.btnCancel)


    def languageChange(self):
        self.setCaption(self.__tr("Edit address"))
        self.cbComment.clear()
        self.cbComment.insertItem(self.__tr("Post address"))
        self.cbComment.insertItem(self.__tr("Office address"))
        self.cbComment.insertItem(self.__tr("Headquarter"))
        self.cbComment.insertItem(self.__tr("External Office"))
        self.lblComment.setText(self.__tr("Comment:"))
        self.gbAddress.setTitle(self.__tr("Complete address:"))
        self.lblTitle.setText(self.__tr("Title:"))
        self.cbCity.clear()
        self.cbCity.setCurrentItem(0)
        self.lblReceiver.setText(self.__tr("Addressee:"))
        self.lblRecComment.setText(self.__tr("Addressee comment:"))
        self.lblStreet.setText(self.__tr("Street or P.O.Box:"))
        self.lblZip.setText(self.__tr("Zip code / city:"))
        self.lblDistrict.setText(self.__tr("District of city:"))
        self.btnOK.setText(self.__tr("&OK"))
        self.btnOK.setAccel(self.__tr("Alt+O"))
        self.btnCancel.setText(self.__tr("&Cancel"))
        self.btnCancel.setAccel(self.__tr("Alt+C"))

        self.lvCountryZone.header().setLabel(0,self.__tr("Country / zone:"))
        self.lvCountryZone.clear()

        Countries = kumula.CountryIntf.getCountries(0)
        for country in Countries:
            cItem = KListViewItemWithID(self.lvCountryZone, country[0])
            cItem.setText(0, country[1])
            Zones = kumula.CountryIntf.getZonesOfCountry(country[0])
            for zone in Zones:
                zItem = KListViewItemWithID(cItem, zone[0])
                zItem.setText(0, zone[1])
        self.lvCountryZone.setSelected(self.lvCountryZone.firstChild(), 1)
        self.lvCountryZone.setCurrentItem(self.lvCountryZone.firstChild())



    def setAllTypes(self):
        print "KAddressDlg.setAllTypes(): Not implemented yet"



    def updateCity(self):
        # Workaround, because QComboBox.clear() clears all - even the currentText...
        currentText = self.cbCity.currentText()
        self.cbCity.clear()
        # ---------------
        cz_item = self.lvCountryZone.currentItem()
        cities = ()
        if (cz_item):
            if (cz_item.parent()):
                # zone is selected
                cities = kumula.CountryIntf.getCitiesFromZone(self.edZip.text(), cz_item.tag)
            else:
                # country is selected
                cities = kumula.CountryIntf.getCitiesFromCountry(self.edZip.text(), cz_item.tag)
        cityCount = 0
        for c in cities:
            cityCount = cityCount + 1
            self.cbCity.insertItem(c[1])

        self.cbCity.setCurrentText(currentText)
        if (cityCount > 0):
            if (currentText == ""):
                self.cbCity.setCurrentText(cities[0][1])
            self.cbCity.popup()



    def __tr(self,s,c = None):
        return qApp.translate("KAddressDlg",s,c)

    def __trUtf8(self,s,c = None):
        return qApp.translate("KAddressDlg",s,c,QApplication.UnicodeUTF8)
