
# for temporary file handling
import os
import tempfile
# for image manipulation
try:
    import Image
except:
    pass

import kumula
from qt import *


class KImageDialog(QDialog):

    def __init__(self, parent, contactid):
        QDialog.__init__(self,parent,"KImageDialog",0,0)
        ImageDialogLayout = QGridLayout(self,1,1,11,6,"ImageDialogLayout")

        self.btnLoad = QPushButton(self,"btnLoad")
        ImageDialogLayout.addWidget(self.btnLoad,2,0)

        self.line3 = QFrame(self,"line3")
        self.line3.setFrameShape(QFrame.HLine)
        self.line3.setFrameShadow(QFrame.Sunken)
        self.line3.setFrameShape(QFrame.HLine)
        ImageDialogLayout.addMultiCellWidget(self.line3,3,3,0,2)

        self.grpTruncation = QButtonGroup(self,"grpTruncation")
        self.grpTruncation.setColumnLayout(0,Qt.Vertical)
        self.grpTruncation.layout().setSpacing(6)
        self.grpTruncation.layout().setMargin(11)
        grpTruncationLayout = QVBoxLayout(self.grpTruncation.layout())
        grpTruncationLayout.setAlignment(Qt.AlignTop)
        self.rdTruncBoth = QRadioButton(self.grpTruncation,"rdTruncBoth")
        self.rdTruncBoth.setChecked(1)
        grpTruncationLayout.addWidget(self.rdTruncBoth)
        self.rdTruncRight = QRadioButton(self.grpTruncation,"rdTruncRight")
        grpTruncationLayout.addWidget(self.rdTruncRight)
        self.rdTruncLeft = QRadioButton(self.grpTruncation,"rdTruncLeft")
        grpTruncationLayout.addWidget(self.rdTruncLeft)
        ImageDialogLayout.addMultiCellWidget(self.grpTruncation,1,2,1,1)

        self.grpRotation = QButtonGroup(self,"grpRotation")
        self.grpRotation.setColumnLayout(0,Qt.Vertical)
        self.grpRotation.layout().setSpacing(6)
        self.grpRotation.layout().setMargin(11)
        grpRotationLayout = QVBoxLayout(self.grpRotation.layout())
        grpRotationLayout.setAlignment(Qt.AlignTop)
        self.rdNoRotation = QRadioButton(self.grpRotation,"rdNoRotation")
        self.rdNoRotation.setChecked(1)
        grpRotationLayout.addWidget(self.rdNoRotation)
        self.rdLeftRotation = QRadioButton(self.grpRotation,"rdLeftRotation")
        grpRotationLayout.addWidget(self.rdLeftRotation)
        self.rdRightRotation = QRadioButton(self.grpRotation,"rdRightRotation")
        grpRotationLayout.addWidget(self.rdRightRotation)
        ImageDialogLayout.addWidget(self.grpRotation,0,1)

        self.btnPreview = QPushButton(self,"btnPreview")
        ImageDialogLayout.addWidget(self.btnPreview,2,2)

        self.frmSource = QFrame(self,"frmSource")
        self.frmSource.setFrameShape(QFrame.StyledPanel)
        self.frmSource.setFrameShadow(QFrame.Sunken)
        frmSourceLayout = QGridLayout(self.frmSource,1,1,11,6,"frmSourceLayout")
        self.pixSource = QLabel(self.frmSource,"pixSource")
        self.pixSource.setMaximumSize(QSize(200,200))
        self.pixSource.setScaledContents(1)
        self.pixSource.setAlignment(QLabel.AlignCenter)
        frmSourceLayout.addWidget(self.pixSource,0,0)
        ImageDialogLayout.addMultiCellWidget(self.frmSource,0,1,0,0)

        self.frmDest = QFrame(self,"frmDest")
        self.frmDest.setFrameShape(QFrame.StyledPanel)
        self.frmDest.setFrameShadow(QFrame.Sunken)
        frmDestLayout = QGridLayout(self.frmDest,1,1,11,6,"frmDestLayout")
        self.pixDest = QLabel(self.frmDest,"pixDest")
        self.pixDest.setMaximumSize(QSize(60,80))
        self.pixDest.setAlignment(QLabel.AlignCenter)
        frmDestLayout.addWidget(self.pixDest,0,0)
        ImageDialogLayout.addMultiCellWidget(self.frmDest,0,1,2,2)

        layout1 = QHBoxLayout(None,0,6,"layout1")
        spacer1 = QSpacerItem(220,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        layout1.addItem(spacer1)

        self.btnSave = QPushButton(self,"btnSave")
        layout1.addWidget(self.btnSave)
        self.btnCancel = QPushButton(self,"btnCancel")
        layout1.addWidget(self.btnCancel)
        ImageDialogLayout.addMultiCellLayout(layout1,4,4,0,2)
        self.languageChange()

        self.resize(QSize(600,330).expandedTo(self.minimumSizeHint()))
        self.clearWState(Qt.WState_Polished)

        self.connect(self.btnLoad,SIGNAL("clicked()"),self.loadImage)
        self.connect(self.btnPreview,SIGNAL("clicked()"),self.generatePreview)
        self.connect(self.btnSave,SIGNAL("clicked()"),self.saveImage)
        self.connect(self.btnCancel,SIGNAL("clicked()"),self.cancelDialog)

        self.setTabOrder(self.btnLoad,self.rdNoRotation)
        self.setTabOrder(self.rdNoRotation,self.rdTruncBoth)
        self.setTabOrder(self.rdTruncBoth,self.btnPreview)
        self.setTabOrder(self.btnPreview,self.btnSave)
        self.setTabOrder(self.btnSave,self.btnCancel)

        self.contactID = contactid
        self.sourceFileName = None
        self.destFileName = ""


    def languageChange(self):
        self.setCaption(self.__tr("Load/save image"))
        self.btnLoad.setText(self.__tr("Load image from file..."))
        self.grpTruncation.setTitle(self.__tr("Truncation"))
        self.rdTruncBoth.setText(self.__tr("from both sides"))
        self.rdTruncRight.setText(self.__tr("from right/bottom"))
        self.rdTruncLeft.setText(self.__tr("from left/top"))
        self.grpRotation.setTitle(self.__tr("Rotation:"))
        self.rdNoRotation.setText(self.__tr("Don't rotate"))
        self.rdLeftRotation.setText(self.__tr("Rotate left"))
        self.rdRightRotation.setText(self.__tr("Rotate right"))
        self.btnPreview.setText(self.__tr("Generate preview"))
        self.pixSource.setText(QString.null)
        self.btnSave.setText(self.__tr("Save image"))
        self.btnCancel.setText(self.__tr("Cancel"))


    def loadImage(self):
        try:
            self.sourceFileName = QFileDialog.getOpenFileName("~/", "All files (*.*)", kumula.MainWin, "LoadImgDlg", "Choose a picture to import")
            if (self.sourceFileName):
                self.pixSource.setPixmap(QPixmap(self.sourceFileName))
        except:
            QMessageBox.warning(kumula.MainWin, "Error", "Can't open image file!", "&OK")


    def generatePreview(self):
        myWidth  = 60
        myHeight = 80

        if (self.destFileName != ""):
            os.unlink(self.destFileName)
        if (self.sourceFileName):
            try:
                im = Image.open(self.sourceFileName.ascii())
                (imWidth,imHeight) = im.size

                # do rotation
                if self.rdNoRotation.isChecked():
                    srcWidth  = imWidth
                    srcHeight = imHeight
                elif self.rdLeftRotation.isChecked():
                    im = im.rotate(90)
                    srcWidth  = imHeight
                    srcHeight = imWidth
                elif self.rdRightRotation.isChecked():
                    im = im.rotate(270)
                    srcWidth  = imHeight
                    srcHeight = imWidth

                # do resizing
                facW = srcWidth  / float(myWidth)
                facH = srcHeight / float(myHeight)
                if (facW < facH):
                    myFactor = facW
                else:
                    myFactor = facH
                destWidth  = int(srcWidth  / myFactor)
                destHeight = int(srcHeight / myFactor)
                im = im.resize((destWidth,destHeight), Image.ANTIALIAS)

                # do truncation
                cropCountW = destWidth  - myWidth
                cropCountH = destHeight - myHeight
                if self.rdTruncBoth.isChecked():
                    cLeft = cropCountW / 2
                    cTop  = cropCountH / 2
                elif self.rdTruncRight.isChecked():
                    cLeft = 0
                    cTop  = 0
                elif self.rdTruncLeft.isChecked():
                    cLeft = cropCountW
                    cTop  = cropCountH
                cRight  = cLeft + myWidth
                cBottom = cTop + myHeight
                im = im.crop((cLeft, cTop, cRight, cBottom))

            except:
                QMessageBox.critical(kumula.MainWin, "Can't create image",
                "Error during resizing/truncating of the image!", "&OK")
                return

            # do saving
            self.destFileName = tempfile.mktemp()
            try:
                im.save(self.destFileName, "JPEG")
            except:
                QMessageBox.critical(kumula.MainWin, "Can't create image",
                "The creation of the temporary image file has failed!", "&OK")
                return
            self.pixDest.setPixmap(QPixmap(self.destFileName))


    def formatHexData(self, data):
        # slow, but save! ;-)
        myHex = ""
        for c in data:
            myHex = myHex + (hex(ord(c))[2:]).zfill(2)
        myHex = "0x" + myHex
        return myHex


    def saveImage(self):
        # Delete old photo first
        # If the user didn't load a photo, the already stored one will be deleted, too.
        # It's not a bug, it's a feature! :-)
        kumula.DB.doDelete("clients_contactphotos", {"contactid":self.contactID}, 1)
        if (self.destFileName != ""):
            myFile = QFile(self.destFileName)
            if (myFile.open(IO_ReadOnly)):
                data = myFile.readBlock(65535)
                myStatement = QString("INSERT INTO clients_contactphotos (contactid,image) VALUES ('")
                myStatement.append(self.contactID)
                myStatement.append("', ")
                myStatement.append(self.formatHexData(data))
                myStatement.append(" );")
                quInsert = kumula.DB.execStatement(myStatement)
                myFile.close()
            os.unlink(self.destFileName)
        self.accept()


    def cancelDialog(self):
        if (self.destFileName != ""):
            os.unlink(self.destFileName)
        self.reject()


    def __tr(self,s,c = None):
        return qApp.translate("ImageDialog",s,c)
