# ============================================================
# plugin.py ("CONTACT")
# plugin for contact management (persons, addresses, comm.data)
# ------------------------------------------------------------
# Copyright (C) 2005 the Kumula Team.
# ============================================================

import os
import tempfile
import kumula
from qt import *
from ClientsView import KClientsView
from ContactPersonDialog import KContactPersonDialog
from AddressDialog import KAddressDialog
from CommunicationsDialog import KCommunicationsDialog
from CommunicationsDialog import CommunicationTypes
from ContactsSettingsDialog import KContactsSettingsDialog
from ImageDialog import KImageDialog
from ClientInterface import KClient

# ------------------------------------------------------------


class cvPlugin(KClientsView):

    # We have to save the temporary filenames (of contact photos) for later cleaning.
    # This is the only reason why this view has to reimplement it's constructor!

    def __init__(self, parent):
        KClientsView.__init__(self, parent)
        self.tempImages = ()

    # Because of Python's Garbage Collector, a destructor wouldn't be called in most cases.
    # So the cleanUp()-method was implemented in KClientsView...

    def cleanUp(self):
        for fileName in self.tempImages:
            os.unlink(fileName)


    # --------------------------------------------------------


    def getTitle(self):
        myTitle = QString("Contacts")
        return myTitle


    def handleAction(self, context, action, id):
        if (context == 'contact'):
            if (action == 'new'):
                self.newContact()
            elif (action == 'edit'):
                self.editContact(id)
            elif (action == 'del'):
                self.delContact(id)
            elif (action == 'picture'):
                self.editPicture(id)
            elif (action == 'email'):
                self.sendEmailTo(id)
            else:
                self.unknownCommand()

        elif (context == 'address'):
            if (action == 'new'):
                self.newAddress()
            elif (action == 'edit'):
                self.editAddress(id)
            elif (action == 'del'):
                self.delAddress(id)
            else:
                self.unknownCommand()

        elif (context == 'communication'):
            if (action == 'new'):
                self.newCommunication()
            elif (action == 'edit'):
                self.editCommunication(id)
            elif (action == 'del'):
                self.delCommunication(id)
            elif (action == 'email'):
                self.sendEmailTo(id)
            elif (action == 'website'):
                self.callBrowser(id)
            else:
                self.unknownCommand()

        else:
            self.unknownCommand()



    def getContent(self, cid):
        self.showContactPersons  = kumula.ProgramSettings.readInt("contacts", "showContactPersons", 1)
        self.showContactPhoto    = kumula.ProgramSettings.readInt("contacts", "showContactPhoto", 0)
        self.showContactFunction = kumula.ProgramSettings.readInt("contacts", "showContactFunction", 1)
        self.showContactEmail    = kumula.ProgramSettings.readInt("contacts", "showContactEmail", 0)
        self.showContactMobile   = kumula.ProgramSettings.readInt("contacts", "showContactMobile", 0)
        self.showCommunications  = kumula.ProgramSettings.readInt("contacts", "showCommunications", 1)
        self.showAddresses       = kumula.ProgramSettings.readInt("contacts", "showAddresses", 1)
        self.showAddressesFull   = kumula.ProgramSettings.readInt("contacts", "showAddressesFull", 0)
        self.showSpacing         = kumula.ProgramSettings.readInt("contacts", "showSpacing", 1)

        content = QString('''
            <html>
            <body bgcolor="#FFFFFF" text="#000000" link="#0000FF" alink="#FF0000" vlink="#0000FF"><br>
            <table border="0" width="100%" cellspacing="5"><tr>
        ''')
        if (self.showContactPersons != 0):
            content.append('<td valign="top" width="50%" rowspan="2">')
            content.append(self.getContactPersons())
            content.append('</td>')
        content.append('<td rowspan="2">&nbsp;</td>')
        if (self.showAddresses != 0):
            content.append('<td valign="top" width="50%">')
            content.append(self.getAddresses())
            content.append('</td>')
        content.append('</tr><tr>')
        if (self.showCommunications != 0):
            content.append('<td valign="top" width="50%">')
            content.append(self.getCommunications())
            content.append('</td>')
        content.append('''
            </tr></table>
            </body>
            </html>
        ''')
        return content


    def configureView(self):
        dlg = KContactsSettingsDialog(kumula.MainWin)
        dlg.cbContactPersons.setChecked(self.showContactPersons)
        dlg.cbContactPhoto.setChecked(self.showContactPhoto)
        dlg.cbContactFunction.setChecked(self.showContactFunction)
        dlg.cbContactEmail.setChecked(self.showContactEmail)
        dlg.cbContactMobile.setChecked(self.showContactMobile)
        dlg.cbCommunications.setChecked(self.showCommunications)
        dlg.cbAddresses.setChecked(self.showAddresses)
        dlg.cbAddressesFull.setChecked(self.showAddressesFull)
        dlg.cbSpacing.setChecked(self.showSpacing)

        if (dlg.exec_loop() == QDialog.Accepted):
            self.showContactPersons  = int(dlg.cbContactPersons.isChecked())
            self.showContactPhoto    = int(dlg.cbContactPhoto.isChecked())
            self.showContactFunction = int(dlg.cbContactFunction.isChecked())
            self.showContactEmail    = int(dlg.cbContactEmail.isChecked())
            self.showContactMobile   = int(dlg.cbContactMobile.isChecked())
            self.showCommunications  = int(dlg.cbCommunications.isChecked())
            self.showAddresses       = int(dlg.cbAddresses.isChecked())
            self.showAddressesFull   = int(dlg.cbAddressesFull.isChecked())
            self.showSpacing         = int(dlg.cbSpacing.isChecked())
            kumula.ProgramSettings.writeInt("contacts", "showContactPersons",  self.showContactPersons)
            kumula.ProgramSettings.writeInt("contacts", "showContactPhoto",    self.showContactPhoto)
            kumula.ProgramSettings.writeInt("contacts", "showContactFunction", self.showContactFunction)
            kumula.ProgramSettings.writeInt("contacts", "showContactEmail",    self.showContactEmail)
            kumula.ProgramSettings.writeInt("contacts", "showContactMobile",   self.showContactMobile)
            kumula.ProgramSettings.writeInt("contacts", "showCommunications",  self.showCommunications)
            kumula.ProgramSettings.writeInt("contacts", "showAddresses",       self.showAddresses)
            kumula.ProgramSettings.writeInt("contacts", "showAddressesFull",   self.showAddressesFull)
            kumula.ProgramSettings.writeInt("contacts", "showSpacing",         self.showSpacing)
            self.reloadContent()


    def moduleMenuItems(self):
#        myItems = ("New contact...", "New address...", "New communication data...", "---", "Birthday list...",)
        myItems = ("New contact...", "New address...", "New communication data...")
        return myItems

    def handleMenuItem(self, nr):
        if (nr == 0):
            self.newContact()
        elif (nr == 1):
            self.newAddress()
        elif (nr == 2):
            self.newCommunication()
        # Birthday list
        elif (nr == 3):
            QMessageBox.information(kumula.MainWin, "Birthday list",
            "Sorry, but the birthday list\nisn't implemented yet... :-(")


    def deleteClientData(self, aClientID):
        kumula.DB.doDelete("clients_communications", {"clientid":aClientID})
        kumula.DB.doDelete("clients_addresses", {"clientid":aClientID})
        # TODO: delete the contact related data: communications and contactphotos!
        kumula.DB.doDelete("clients_contacts", {"clientid":aClientID})



#############################################################################
# FUNCTIONS FOR CREATING, CHANGING AND DELETING ENTRIES
#############################################################################


    def newContact(self):
        dlg = KContactPersonDialog(kumula.MainWin)
        dlg.edTitle.setText("")
        dlg.edName.setText("")
        dlg.edForename.setText("")
        dlg.edFunction.setText("")
        dlg.edBirthday.setDate(QDate(1970,01,01))
        dlg.setAddressesFromContact(self.cID, "")
        dlg.setCommunicationsFromContact("", 0)
        if dlg.exec_loop() == QDialog.Accepted:
            kumula.DB.doInsert("clients_contacts", {"clientid":self.cID, "title":dlg.edTitle.text(), "name":dlg.edName.text(), "forename":dlg.edForename.text(), "function":dlg.edFunction.text(), "division":dlg.edDivision.text(), "birthday":dlg.edBirthday.date().toString("yyyy-MM-dd"), "addressid":dlg.lvAddress.currentItem().tag})
            myID = kumula.DB.lastAutoID()
            if (myID > 0):
                dlg.writeCommunicationsToDB(self.cID, "%i" % myID)
            self.reloadContent()



    def editContact(self, id):
        quContact = kumula.DB.doSelect("clients_contacts", ("title","name","forename","function","division","birthday"), "", {"clientid":self.cID, "id":id}, 1)
        dlg = KContactPersonDialog(kumula.MainWin)
        while quContact.next():
            dlg.edTitle.setText(quContact.value(0).toString())
            dlg.edName.setText(quContact.value(1).toString())
            dlg.edForename.setText(quContact.value(2).toString())
            dlg.edFunction.setText(quContact.value(3).toString())
            dlg.edDivision.setText(quContact.value(4).toString())
            dlg.edBirthday.setDate(QDate.fromString(quContact.value(5).toString(), Qt.ISODate))
            dlg.setAddressesFromContact(self.cID, id)
            dlg.setCommunicationsFromContact(self.cID, id)
        if (dlg.exec_loop() == QDialog.Accepted):
            kumula.DB.doUpdate("clients_contacts", {"title":dlg.edTitle.text(), "name":dlg.edName.text(), "forename":dlg.edForename.text(), "function":dlg.edFunction.text(), "division":dlg.edDivision.text(), "birthday":dlg.edBirthday.date().toString("yyyy-MM-dd"), "addressid":dlg.lvAddress.currentItem().tag}, {"id":id}, 1)
            dlg.writeCommunicationsToDB(self.cID, id)
            self.reloadContent()


    def delContact(self, id):
        result = QMessageBox.warning(kumula.MainWin, "Question",
            "Do you really want to delete\nthis contact person?",
            "&Delete", "&Cancel")
        if (result == 0):
            kumula.DB.doDelete("clients_communications", {"clientid":self.cID, "contactid":id})
            kumula.DB.doDelete("clients_contactphotos", {"contactid":id}, 1)
            kumula.DB.doDelete("clients_contacts", {"id":id}, 1)
            self.reloadContent()


    def newAddress(self):
        myClient = KClient(self.cID)
        dlg = KAddressDialog()
        dlg.setCaption("New address")
        dlg.edTitle.clear()
        dlg.edReceiver.setText(myClient.clientName)
        dlg.edRecComment.clear()
        dlg.edStreet.clear()
        dlg.edZip.clear()
        dlg.edDistrict.clear()
        if dlg.exec_loop() == QDialog.Accepted:
            ZoneOrCountry = ""
            myZone = ""
            if dlg.lvCountryZone.currentItem():
                ZoneOrCountry = "C"
                myZone = dlg.lvCountryZone.currentItem().tag
                if dlg.lvCountryZone.currentItem().parent():
                    ZoneOrCountry = "Z"
            kumula.DB.doInsert("clients_addresses", {"clientid":self.cID, "comment":dlg.cbComment.currentText(), "zone":myZone, "zone_country":ZoneOrCountry, "zip":dlg.edZip.text(), "city":dlg.cbCity.currentText(), "district":dlg.edDistrict.text(), "address":dlg.edStreet.text(), "title":dlg.edTitle.text(), "addressee":dlg.edReceiver.text(), "adcomment":dlg.edRecComment.text()})
            self.reloadContent()

    
    def markZone(self, item, id, zone_country, addrDlg):
        if (item):
            found = 0
            if (zone_country == "Z"):
                # let's check the zone id
                if (item.parent()):
                    # item must have a parent, else it would be no zone
                    if (item.tag == id):
                        found = 1
            else:
                # let's check for a country id
                if (item.parent()):
                    # item mustn't have a parent, else it would be a zone
                    pass
                else:
                    if (item.tag == id):
                        found = 1
            if (found == 1):
                addrDlg.lvCountryZone.setSelected(item, 1)
                addrDlg.lvCountryZone.setCurrentItem(item)
            else:
                self.markZone(item.firstChild(), id, zone_country, addrDlg)
                self.markZone(item.nextSibling(), id, zone_country, addrDlg)


    def editAddress(self, id):
        dlg = KAddressDialog()
        quAddress = kumula.DB.doSelect("clients_addresses", ("comment","zone","zone_country","zip","city","district","address","title","addressee","adcomment"), "", {"id":id}, 1)
        while quAddress.next():
            dlg.setCaption("Edit address")
            dlg.cbComment.setCurrentText(quAddress.value(0).toString())
            dlg.edZip.setText(quAddress.value(3).toString())
            dlg.cbCity.setCurrentText(quAddress.value(4).toString())
            dlg.edDistrict.setText(quAddress.value(5).toString())
            dlg.edStreet.setText(quAddress.value(6).toString())
            dlg.edTitle.setText(quAddress.value(7).toString())
            dlg.edReceiver.setText(quAddress.value(8).toString())
            dlg.edRecComment.setText(quAddress.value(9).toString())
            self.markZone(dlg.lvCountryZone.firstChild(), quAddress.value(1).toString(), quAddress.value(2).toString(), dlg)
        if dlg.exec_loop() == QDialog.Accepted:
            ZoneOrCountry = ""
            myZone = ""
            if dlg.lvCountryZone.currentItem():
                ZoneOrCountry = "C"
                myZone = dlg.lvCountryZone.currentItem().tag
                if dlg.lvCountryZone.currentItem().parent():
                    ZoneOrCountry = "Z"
            kumula.DB.doUpdate("clients_addresses", {"comment":dlg.cbComment.currentText(), "zone":myZone, "zone_country":ZoneOrCountry, "zip":dlg.edZip.text(), "city":dlg.cbCity.currentText(), "district":dlg.edDistrict.text(), "address":dlg.edStreet.text(), "title":dlg.edTitle.text(), "addressee":dlg.edReceiver.text(), "adcomment":dlg.edRecComment.text()}, {"id":id}, 1)
            self.reloadContent()
    
    
    def delAddress(self, id):
        result = QMessageBox.warning(kumula.MainWin, "Question",
            "Do you really want to delete\nthis address?",
            "&Delete", "&Cancel")
        if (result == 0):
            kumula.DB.doDelete("clients_addresses", {"id":id}, 1)
            self.reloadContent()


    
    def newCommunication(self):
        dlg = KCommunicationsDialog()
        dlg.cbCommType.setCurrentItem(0)
        dlg.edValue.setText("")
        if dlg.exec_loop() == QDialog.Accepted:
            myTypeStr = QString("")
            myTypeStr.setNum(dlg.cbCommType.currentItem())
            kumula.DB.doInsert("clients_communications", {"clientid":self.cID, "type":myTypeStr, "value":dlg.edValue.text()})
            self.reloadContent()



    def editCommunication(self, id):
        quComm = kumula.DB.doSelect("clients_communications", ("type","value"), "", {"id":id})
        while quComm.next():
            myType  = quComm.value(0).toInt()
            myValue = quComm.value(1).toString()
        dlg = KCommunicationsDialog()
        dlg.cbCommType.setCurrentItem(myType)
        dlg.edValue.setText(myValue)
        if dlg.exec_loop() == QDialog.Accepted:
            myTypeStr = QString("")
            myTypeStr.setNum(dlg.cbCommType.currentItem())
            kumula.DB.doUpdate("clients_communications", {"type":myTypeStr, "value":dlg.edValue.text()}, {"id":id}, 1)
            self.reloadContent()



    def delCommunication(self, id):
        result = QMessageBox.warning(kumula.MainWin, "Question",
            "Do you really want to delete\nthis communication entry?",
            "&Delete", "&Cancel")
        if (result == 0):
            kumula.DB.doDelete("clients_communications", {"id":id}, 1)
            self.reloadContent()



#############################################################################
# FUNCTIONS FOR DISPLAYING THE VALUES
#############################################################################


    def htmlTitle(self, title, type):
        myTitle = QString('<table border="0" width="100%" cellpadding="2" cellspacing="0" bgcolor="#AAAAAA">')
        myTitle.append('<tr><td><font color="#FFFFFF"><b>')
        myTitle.append(title)
        myTitle.append('</b></font></td><td align="right">')
        myTitle.append('<a href="')
        myTitle.append(self.createURL(type, "new", "-1"))
        myTitle.append('"><img src="')
        myTitle.append(kumula.ProgramPath)
        myTitle.append('/graphics/new.gif"></a>')
        myTitle.append('</td></tr></table><br><br>')
        return myTitle


    def htmlUnit(self, title, content, type):
        myUnit = QString(self.htmlTitle(title, type))
        myUnit.append('<table border="0" width="100%" cellpadding="2" cellspacing="0">')
        myUnit.append(content)
        myUnit.append('</table>')
        return myUnit


    def htmlEntry(self, header, additional, type, index, picture=''):
        myEntry = QString('<tr>')
        if (picture != ''):
            myEntry.append('<td><a href="')
            myEntry.append(self.createURL(type, "picture", index))
            myEntry.append('"><img src="')
            myEntry.append(picture)
            myEntry.append('" width="60" height="80"></a></td>')
            columns = "3"
        else:
            columns = "2"
        myEntry.append('<td><b>')
        myEntry.append(header)
        myEntry.append('</b><br>')
        myEntry.append(additional)
        myEntry.append('</td><td align="right"><a href="')
        myEntry.append(self.createURL(type, "edit", index))
        myEntry.append('"><img src="')
        myEntry.append(kumula.ProgramPath)
        myEntry.append('/graphics/edit.gif"></a><a href="')
        myEntry.append(self.createURL(type, "del", index))
        myEntry.append('"><img src="')
        myEntry.append(kumula.ProgramPath)
        myEntry.append('/graphics/del.gif"></a></td></tr>')
        if (self.showSpacing == 1):
            myEntry.append('<tr><td colspan="')
            myEntry.append(columns)
            myEntry.append('">&nbsp;</td></tr>')
        return myEntry


    def getPictureFile(self, contactid):
        quData = kumula.DB.doSelect("clients_contactphotos", ("image",), "", {"contactid":contactid})
        if quData.next():
            destFileName = tempfile.mktemp()
            myFile = QFile(destFileName)
            if (myFile.open(IO_WriteOnly)):
                myFile.writeBlock(quData.value(0).toByteArray())
                myFile.close()
            self.tempImages = self.tempImages + (destFileName,)
            return QString(destFileName)
        else:
            myPath = QString(kumula.ProgramPath)
            myPath.append("/graphics/nocontact.jpg")
            return myPath


    def getContactPersons(self):
        # First, delete old photos
        for fileName in self.tempImages:
            os.unlink(fileName)
        self.tempImages = ()

        myContent = QString('')
        quContacts = kumula.DB.doSelect("clients_contacts", ("id","name","forename","function","division"), "name", {"clientid":self.cID})
        count = 0
        while quContacts.next():
            count = count + 1
            myHeader = QString(quContacts.value(1).toString())
            myHeader.append(' ')
            myHeader.append(quContacts.value(2).toString())
            myAdditional = QString("")
            if (self.showContactFunction != 0):
                myAdditional.append(quContacts.value(3).toString())
                if (quContacts.value(3).toString() != "") and (quContacts.value(4).toString() != ""):
                    myAdditional.append(", ")
                myAdditional.append(quContacts.value(4).toString())
                myAdditional.append("<br>")
            if (self.showContactMobile != 0):
                quMobile = kumula.DB.doSelect("clients_communications", ("value",), "id", {"contactid":quContacts.value(0).toString(), "type":"2"})
                while quMobile.next():
                    myAdditional.append(quMobile.value(0).toString())
                    myAdditional.append('<br>')
            if (self.showContactEmail != 0):
                quEmail = kumula.DB.doSelect("clients_communications", ("id","value"), "id", {"contactid":quContacts.value(0).toString(), "type":"4"})
                while quEmail.next():
                    myAdditional.append('<a href="')
                    myAdditional.append(self.createURL("contact", "email", quEmail.value(0).toString()))
                    myAdditional.append('">')
                    myAdditional.append(quEmail.value(1).toString())
                    myAdditional.append('</a><br>')
            # the last "<br>" should be removed...
            myAdditional.remove(myAdditional.length()-4, 4)
            if (self.showContactPhoto == 0):
                myContact = self.htmlEntry(myHeader, myAdditional, 'contact', quContacts.value(0).toString())
            else:
                myContact = self.htmlEntry(myHeader, myAdditional, 'contact', quContacts.value(0).toString(), self.getPictureFile(quContacts.value(0).toString()))
            myContent.append(myContact)
        if (count == 0):
            myContent.append("<tr><td>No contact persons defined.</td></tr>")
        myUnit = self.htmlUnit('Contact&nbsp;persons', myContent, 'contact')
        return myUnit



    def getAddresses(self):
        myContent = QString('')
        quAddresses = kumula.DB.doSelect("clients_addresses", ("id","comment","zip","city","district","address","zone","zone_country"), "comment", {"clientid":self.cID})
        count = 0
        while quAddresses.next():
            count = count + 1
            myAdditional = QString(quAddresses.value(5).toString())
            myAdditional.append('<br>')
            myAdditional.append(quAddresses.value(2).toString())
            myAdditional.append('&nbsp;')
            myAdditional.append(quAddresses.value(3).toString())

            if (quAddresses.value(4).toString() != ''):
                myAdditional.append('-')
                myAdditional.append(quAddresses.value(4).toString())

            myZoneName = ""
            myCountryName = ""
            countryID = quAddresses.value(6).toString()
            zoneID = countryID
            zoneOrCountry = quAddresses.value(7).toString()
            if (zoneOrCountry == "Z"):
                myZoneName = kumula.CountryIntf.getZoneName(zoneID)
                country = kumula.CountryIntf.getCountryFromZone(zoneID, 0)
            else:
                country = kumula.CountryIntf.getCountry(countryID, 0)
            countryID = country[0]
            myCountryName = country[1]

            if (myCountryName != ""):
                myAdditional.append('<br>')
                myAdditional.append(myCountryName)
                if (myZoneName != ""):
                    myAdditional.append('/')
                    myAdditional.append(myZoneName)

            myTitle = QString(quAddresses.value(1).toString())
            myTitle.append(":")
            myAddress = self.htmlEntry(myTitle, myAdditional, 'address', quAddresses.value(0).toString())

            myContent.append(myAddress)
        if (count == 0):
            myContent.append("<tr><td>No addresses defined.</td></tr>")
        myUnit = self.htmlUnit('Addresses', myContent, 'address')
        return myUnit



    def getCommunications(self):
        myContent = QString('')
        quComms = kumula.DB.doSelect("clients_communications", ("id","type","value"), "type", {"clientid":self.cID, "contactid":"0"})
        count = 0
        while quComms.next():
            count = count + 1
            myItems = QStringList.split(QChar(";"), CommunicationTypes)
            myType  = quComms.value(1).toInt()
            myHeader = QString(myItems[myType])
            myHeader.append(':')
            myAdditional = QString('')
            if (myType == 4):
                myAdditional.append('<a href="')
                myAdditional.append(self.createURL("communication", "email", quComms.value(0).toString()))
                myAdditional.append('">')
            if (myType == 3):
                myAdditional.append('<a href="')
                myAdditional.append(self.createURL("communication", "website", quComms.value(0).toString()))
                myAdditional.append('">')
            myAdditional.append(quComms.value(2).toString())
            if (myType == 4):
                myAdditional.append('</a>')
            myComm = self.htmlEntry(myHeader, myAdditional,
                        'communication', quComms.value(0).toString())
            myContent.append(myComm)
        if (count == 0):
            myContent.append("<tr><td>No communication data defined.</td></tr>")
        myUnit = self.htmlUnit('Communication&nbsp;data', myContent, 'communication')
        return myUnit


    def editPicture(self, id):
        # Check the availability of the Python Imaging Library (PIL) from www.PythonWare.com!!!
        try:
            import Image
        except:
            result = QMessageBox.critical(kumula.MainWin, "Library missing",
                "To assign a picture to a contact person, you need to have the <b>Python Imaging Library (PIL)</b> installed.",
                "&OK")
            return
        dlg = KImageDialog(kumula.MainWin, id)
        if (dlg.exec_loop() == QDialog.Accepted):
            self.reloadContent()



    def sendEmailTo(self, id):
        quMail = kumula.DB.doSelect("clients_communications", ("value",), "value", {"id":id}, 1)
        while quMail.next():
            myParam = QString("mailto:")
            myParam.append(quMail.value(0).toString())
            kumula.runExternalApp(kumula.AppEmail, myParam)


    def callBrowser(self, id):
        quWeb = kumula.DB.doSelect("clients_communications", ("value",), "value", {"id":id}, 1)
        while quWeb.next():
            kumula.runExternalApp(kumula.AppBrowser, quWeb.value(0).toString())

