# ============================================================
# plugin.py ("TICKETS")
# plugin for a internal, client-specific message board system.
# ------------------------------------------------------------
# Copyright (C) 2004 the Kumula Team.
# ============================================================

import kumula
from qt import *
from ClientsView import KClientsView
from TicketDialog import KTicketDialog
from TicketSettingsDialog import KTicketSettingsDialog

# ------------------------------------------------------------


class cvPlugin(KClientsView):


    def getTitle(self):
        myTitle = QString("Tickets")
        return myTitle


    def handleAction(self, context, action, id):
        if (context == 'message'):        
            if (action == 'show'):
                self.showMessage(id)
            elif (action == 'new'):
                self.postNewTicket()
            elif (action == 'reply'):
                self.postReply()
            elif (action == 'index'):
                self.reloadContent()
            else:
                self.unknownCommand()
        else:
            self.unknownCommand()


    def getContent(self, cid):
        self.tID = "0"
        content = QString('''
            <html>
            <body bgcolor="#FFFFFF" text="#000000" link="#0000FF" alink="#FF0000" vlink="#0000FF">
            <br>
            <table border="0" width="100%" cellspacing="5">
            <tr>
                <td valign="top">
                ''')
        content.append(self.htmlButton("New&nbsp;ticket", self.createURL("message", "new", "-1")))
        content.append(self.getThreads("0"))
        content.append('''
                </td>
            </tr>
            </table>
            </body>
            </html>
        ''')
        return content


    def configureView(self):
        dlg = KTicketSettingsDialog(kumula.MainWin)
        self.showCompleteThread = kumula.ProgramSettings.readInt("tickets", "showCompleteThread", 1)
        if (self.showCompleteThread == 1):
            dlg.rbComplete.setChecked(1)
        else:
            dlg.rbFollowUps.setChecked(1)

        if (dlg.exec_loop() == QDialog.Accepted):
            if (dlg.rbComplete.isChecked()):
                kumula.ProgramSettings.writeInt("tickets", "showCompleteThread", 1)
            else:
                kumula.ProgramSettings.writeInt("tickets", "showCompleteThread", 0)


    def moduleMenuItems(self):
        myItems = ("Post new ticket...",)
        return myItems

    def handleMenuItem(self, nr):
        if (nr == 0):
            self.postNewTicket()



    def deleteClientData(self, aClientID):
        kumula.DB.doDelete("clients_tickets", {"clientid":aClientID})



#############################################################################
# FUNCTIONS FOR CREATING, CHANGING AND DELETING ENTRIES
#############################################################################


    def postNewTicket(self):
        dlg = KTicketDialog(kumula.MainWin)
        dlg.edSubject.setText("")
        dlg.edMessage.setText("")
        if (dlg.exec_loop() == QDialog.Accepted):
            kumula.DB.doInsert("clients_tickets", {"parentid":"0", "clientid":self.cID, "author":kumula.UserName, "time":QDateTime().currentDateTime().toString(Qt.ISODate), "subject":dlg.edSubject.text(), "body":dlg.edMessage.text()})
            lastID = kumula.DB.lastAutoID()
            if (lastID > 0):
                self.showMessage("%i" % lastID)
            else:
                self.reloadContent()


    def postReply(self):
        dlg = KTicketDialog(kumula.MainWin)
        mySubject = QString("Re: ")
        mySubject.append(self.tSubject)
        dlg.edSubject.setText(mySubject)
        dlg.edMessage.setText("")
        dlg.edMessage.setFocus()
        if (dlg.exec_loop() == QDialog.Accepted):
            kumula.DB.doInsert("clients_tickets", {"parentid":self.tID, "clientid":self.cID, "author":kumula.UserName, "time":QDateTime().currentDateTime().toString(Qt.ISODate), "subject":dlg.edSubject.text(), "body":dlg.edMessage.text()})
            lastID = kumula.DB.lastAutoID()
            if (lastID > 0):
                self.showMessage("%i" % lastID)
            else:
                self.reloadContent()


#############################################################################
# FUNCTIONS FOR DISPLAYING THE VALUES
#############################################################################


    def getThreads(self, parentid):
        count = 0
        myContent = QString("")
        quThreads = kumula.DB.doSelect("clients_tickets", ("id","subject","author","time"), "time", {"clientid":self.cID, "parentid":parentid})
        while quThreads.next():
            count = count + 1
            if (count == 1):
                myContent.append("<ul>")
            myContent.append("<li>")
            myContent.append(self.formatThreadLine(quThreads.value(0).toString(), quThreads.value(1).toString(), quThreads.value(2).toString(), quThreads.value(3).toString()))
            myContent.append(self.getThreads(quThreads.value(0).toString()))
            myContent.append("</li>")
        if (count > 0):
            myContent.append("</ul>")
        return myContent



    def showMessage(self, tID):
        self.showCompleteThread = kumula.ProgramSettings.readInt("tickets", "showCompleteThread", 1)
        self.tID = tID
        quThread = kumula.DB.doSelect("clients_tickets", ("subject","author","time","body"), "", {"id":tID})
        while quThread.next():
            mySubject = quThread.value(0).toString()
            myAuthor  = quThread.value(1).toString()
            myTime    = quThread.value(2).toString()
            myBody    = quThread.value(3).toString()
            myBody.replace(QChar('\n'), "<br>")
            self.tSubject = mySubject

        content = QString('''
            <html>
            <body bgcolor="#FFFFFF" text="#000000" link="#0000FF" alink="#FF0000" vlink="#0000FF">
            <br>
            <table border="0" width="100%" cellspacing="5">
            <tr>
                <td valign="top" colspan="2"><h2>
                ''')
        content.append(mySubject)
        content.append("</h2></td></tr><tr><td valign='top'><ul><li><em>From</em>: ")
        content.append(myAuthor)
        content.append("</li><li><em>Date</em>: ")
        content.append(myTime)
        content.append("</li></ul></td><td valign='top' align='right'>")
        content.append(self.htmlButton("Post&nbsp;reply", self.createURL("message", "reply", tID)))
        content.append("</td></tr><tr><td colspan='2'><hr><tt>")
        content.append(myBody)
        content.append("</tt><hr></td></tr><tr><td valign='top'>")

        if (self.showCompleteThread == 1):
            # shows the complete thread
            content.append("<br><strong>This ticket's entrys:</strong>")
            mainThreadID = tID
            pID = self.getParentID(mainThreadID)
            while (pID != "0"):
                pID = self.getParentID(mainThreadID)
                if (pID != "0"):
                    mainThreadID = pID

            quThreads = kumula.DB.doSelect("clients_tickets", ("subject","author","time"), "time", {"id":mainThreadID})
            while quThreads.next():
                content.append("<ul><li>")
                content.append(self.formatThreadLine(mainThreadID, quThreads.value(0).toString(), quThreads.value(1).toString(), quThreads.value(2).toString()))
                content.append(self.getThreads(mainThreadID))
                content.append("</li></ul>")
        else:
            # shows only following answers
            followups = self.getThreads(tID)
            if (followups != ""):
                content.append("<br><strong>Follow-ups:</strong>")
                content.append(followups)

        content.append("</td><td valign='top' align='right'>")
        content.append(self.htmlButton("Show&nbsp;all&nbsp;tickets", self.createURL("message", "index", "-1")))
        content.append('''
                </td>
            </tr>
            </table>
            </body>
            </html>
        ''')
        self.setText(content)


    def formatThreadLine(self, id, subject, author, time):
        myLine = QString("<strong>")
        if (id != self.tID):
            myLine.append("<a href='")
            myLine.append(self.createURL("message", "show", id))
            myLine.append("'>")
        myLine.append(subject)
        if (id != self.tID):
            myLine.append("</a>")
        myLine.append("</strong>, <em>")
        myLine.append(author)
        myLine.append(" (")
        myLine.append(time)
        myLine.append(")</em>")
        return myLine


    def getParentID(self, id):
        quThread = kumula.DB.doSelect("clients_tickets", ("parentid",), "", {"id":id})
        while quThread.next():
            pID = quThread.value(0).toString()
        return pID


    def htmlButton(self, title, url):
        myTitle = QString('<table border="0" cellpadding="2" cellspacing="0" bgcolor="#AAAAAA">')
        myTitle.append('<tr><td><td><a href="')
        myTitle.append(url)
        myTitle.append('"><font color="#FFFFFF"><b>')
        myTitle.append(title)
        myTitle.append('</b></font>')
        myTitle.append('</td></tr></table>')
        return myTitle


