

import kumula
from qt import *
from SideBar import KSideBar
from ConfigParser import *
from KumulaMainWindow import KKumulaMainWindow,KAction

import sys
import os

# ------------------------------------------------------------


class KMainWindow(KKumulaMainWindow):

    def getProgramName(self):
        return "Configurator"

    def getProgramVersion(self):
        return "1.0"


    def initSettings(self):
        KKumulaMainWindow.initSettings(self)
        self.loadWidgets()
        self.handleReset()


    def createUI(self):
        KKumulaMainWindow.createUI(self)
#        no, we don't need a statusbar!
#        self.statusBar()

        self.setCentralWidget(QWidget(self, "qt_central_widget"))
        myLayout = QHBoxLayout(self.centralWidget())

        self.sideBar = KSideBar(self.centralWidget(), "SideBar")
        myLayout.addWidget(self.sideBar)

        self.widgetFrame = QFrame(self.centralWidget())
        self.widgetFrame.setFrameShape(QFrame.NoFrame)
        widgetFrameLayout = QVBoxLayout(self.widgetFrame, 10, 10)
        myLayout.addWidget(self.widgetFrame)

        self.titleFrame = QFrame(self.widgetFrame,"titleFrame")
        self.titleFrame.setSizePolicy(QSizePolicy(QSizePolicy.Preferred,QSizePolicy.Fixed,0,0,self.titleFrame.sizePolicy().hasHeightForWidth()))
        self.titleFrame.setFrameShape(QFrame.NoFrame)
        self.titleFrame.setFrameShadow(QFrame.Plain)
        titleFrameLayout = QGridLayout(self.titleFrame,1,1,11,6,"titleFrameLayout")

        self.titlePixmap = QLabel(self.titleFrame,"titlePixmap")
        self.titlePixmap.setScaledContents(0)
        titleFrameLayout.addMultiCellWidget(self.titlePixmap,0,1,0,0)

        self.titleLabel = QLabel(self.titleFrame,"titleLabel")
        titleLabel_font = QFont(self.titleLabel.font())
        titleLabel_font.setPointSize(16)
        titleLabel_font.setBold(1)
        self.titleLabel.setFont(titleLabel_font)
        titleFrameLayout.addWidget(self.titleLabel,0,1)
        self.titleDesc = QLabel(self.titleFrame,"titleDesc")
        titleFrameLayout.addWidget(self.titleDesc,1,1)

        titleSpacer = QSpacerItem(220,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        titleFrameLayout.addMultiCell(titleSpacer,0,1,2,2)

        widgetFrameLayout.addWidget(self.titleFrame)

        self.widgetStack = QWidgetStack(self.widgetFrame)
        self.widgetStack.setFrameShape(QFrame.StyledPanel)
        self.widgetStack.setFrameShadow(QFrame.Raised)

        # insert loaded widgets in the widgetStack
        firstID = -99
        for myWidget in self.widgetTuple:
            myID = self.widgetStack.addWidget(myWidget)
            myTitle = myWidget.caption()
            self.sideBar.insertButton(myTitle, myWidget.icon(), myID)
            self.widgetByID[myID] = myWidget
            if (firstID == -99):
                firstID = myID
        self.sideBar.insertSpacer()
        self.connect(self.sideBar, PYSIGNAL("selectionChanged()"), self.updateView)
        self.sideBar.selectButton(firstID)

        widgetFrameLayout.addWidget(self.widgetStack)
        # load the values from the config file
        self.handleReset()

        self.resize(QSize(500,400).expandedTo(self.minimumSizeHint()))
        self.clearWState(Qt.WState_Polished)





    def createMenuBar(self):
        KKumulaMainWindow.createMenuBar(self)

        # initialize Actions
        self.actApply  = KAction(self, "apply",  "&Apply",     self.handleApply, "Apply all settings to your config file.")
        self.actReset  = KAction(self, "reset",  "&Reset",     self.handleReset, "Reset all settings to the stored values from your config file.")
        self.actImport = KAction(self, "import", "&Import...", None,             "Import settings from an external file.")
        self.actExport = KAction(self, "export", "E&xport...", None,             "Export settings to an external file.")

        # create Menu Structure
        self.menuSettings = self.makePopupMenu()
        self.actApply.addToMenuBar(self.menuSettings)
        self.actReset.addToMenuBar(self.menuSettings)
#        self.menuSettings.insertSeparator()
#        self.actImport.addToMenuBar(self.menuSettings)
#        self.actExport.addToMenuBar(self.menuSettings)
        self.MenuBar.insertItem(QString("&Settings"), self.menuSettings, 0)



    def createToolBar(self):
        pass



    def closeEvent(self, event):
        KKumulaMainWindow.closeEvent(self, event)

        if (self.settingsWritten == 0):
            if (QMessageBox.question(self, "Question",
                "Any changes were <b>not</b> saved!<br>Do you really wish to quit this program?",
                QMessageBox.Yes, QMessageBox.No) == QMessageBox.Yes):
                event.accept()
            else:
                event.ignore()
        else:
            try:
                myCaller = kumula.ProgParams["caller"]
            except:
                myCaller = QString("")

            if (myCaller == ""):
                pass
            else:
                myMessage = QString("Do you want to start <b>")
                myMessage.append(myCaller)
                myMessage.append("</b> again?")
                if (QMessageBox.question(self, "Question", myMessage,
                    QMessageBox.Yes, QMessageBox.No) == QMessageBox.Yes):
                    kumula.runKumulaApp(myCaller, {})
            event.accept()



    def updateView(self):
        self.widgetStack.raiseWidget(self.sideBar.currentButton())
        curWidget = self.widgetStack.visibleWidget()
        self.titleLabel.setText(curWidget.caption())
        self.titleDesc.setText(curWidget.getDescription())
        self.titlePixmap.setPixmap(curWidget.icon())


    def handleImport(self):
        pass

    def handleExport(self):
        pass


    def handleApply(self):
        myCfgFile = QString(kumula.UserPath)
        myCfgFile.append("/.kumularc")
        try:
            cfg = open(myCfgFile.latin1(), 'w')
            for myWidget in self.widgetTuple:
                mySection = myWidget.getSectionName()
                cfg.write('[' + mySection + ']\n')
                myValues = myWidget.getValues()
                for key in myValues:
                    # safe concanete for Python and Qt Strings, also for Integers
                    myLine = QString(key)
                    myLine.append("=")
                    myLine.append(str(myValues[key]))
                    myLine.append("\n")
                    cfg.write(myLine.ascii())
                cfg.write('\n')
            self.settingsWritten = 1
            QMessageBox.information(kumula.MainWin, "Information", "Configuration file<br>was saved successfully.")
        except:
            QMessageBox.critical(kumula.MainWin, "Error", "Writing configuration file<br>was <b>not</b> successful!")


    def handleReset(self):
        # first, open the file ~/.kumularc
        myCfgFile = QString(kumula.UserPath)
        myCfgFile.append("/.kumularc")
        try:
            self.ConfigFile = ConfigParser()
            self.ConfigFile.read(myCfgFile.latin1())
        except:
            pass

        # load the stored values from the config file
        self.origValues = {}
        for myWidget in self.widgetTuple:
            mySection = myWidget.getSectionName()
            self.origValues[mySection] = {}
            for key in myWidget.getValues():
                try:
                    self.origValues[mySection][key] = self.ConfigFile.get(mySection, key)
                except:
                    self.origValues[mySection] = myWidget.getDefaultValues()
            myWidget.setValues(self.origValues[mySection])
        self.settingsWritten = 0



    def loadWidgets(self):
        self.widgetByID = {}
        self.widgetTuple = ()

        myViewerDir = os.path.join(os.path.dirname(sys.argv[0]), "..", "lib", "configurator")
        # get list of all "widget_" files in KUMULADIR/lib/configurator
        osList = os.listdir(myViewerDir)
        osList.sort()
        for name in osList:
            myFileName = QString(name)
            if (myFileName.left(7) == "widget_"):
                myFile = os.path.join(myViewerDir, name)

                myPlugin = open(myFile, "r")
                # ...compile it...
                exec myPlugin in locals()
                myPlugin.close()
                # ...and add it to our tuple
                myWidget = ConfigWidget(self)
                self.widgetTuple = self.widgetTuple + (myWidget,)


