# ============================================================
# MainWindow.py
# main window for the user management program.
# ------------------------------------------------------------
# Copyright (C) 2005 the Kumula Team.
# ============================================================

import kumula
from qt import *
from SideBar import KSideBar
from SpecialItems import KListViewItemWithID
from KumulaMainWindow import KKumulaMainWindow,KAction
from UserDialog import KUserDialog

# ------------------------------------------------------------


class KMainWindow(KKumulaMainWindow):

    def getProgramName(self):
        return "Users"

    def getProgramVersion(self):
        return "1.0"


    def createUI(self):
        KKumulaMainWindow.createUI(self)
        self.statusBar()
        # initalize user interface

        self.setCentralWidget(QWidget(self, "qt_central_widget"))
        myLayout = QHBoxLayout(self.centralWidget())

        self.splitter = QSplitter(self.centralWidget(), "splitter")
        self.splitter.setOrientation(Qt.Horizontal)

        self.userList = QListView(self.splitter)
        self.userList.addColumn("Username")
        self.userList.addColumn("ID")
        self.userList.addColumn("Surename")
        self.userList.addColumn("Forename")
        self.userList.setAllColumnsShowFocus(1)
        self.userList.setResizeMode(QListView.LastColumn)

        self.userManagement = QTabWidget(self.splitter)
        self.userManagement.setTabPosition(QTabWidget.Bottom)
        self.userManagement.insertTab(QFrame(self), "Rights")
        self.userManagement.insertTab(QFrame(self), "User data")
        self.userManagement.setEnabled(0)

        myLayout.addWidget(self.splitter)

#        self.connect(self.folderView, PYSIGNAL("selectionChanged()"), self.updateUserList)
#        self.connect(self.userList, SIGNAL("doubleClicked()"), self.slotUserListDoubleClick)
#        self.folderView.selectButton(self.lastFileTypes)
        self.updateUserList()

        self.resize(QSize(600,482).expandedTo(self.minimumSizeHint()))
        self.clearWState(Qt.WState_Polished)



    def createMenuBar(self):
        KKumulaMainWindow.createMenuBar(self)

        # initialize Settings
        self.lastUser = kumula.ProgramSettings.readInt("view", "lastUser", 0)

        # initialize Actions
        self.actUserNew       = KAction(self, "user_new",       "&New...",                      self.handleUserNew,    "Create a new user.")
        self.actUserEdit      = KAction(self, "user_edit",      "&Edit...",                     self.handleUserEdit,   "Change the data of the selected user.")
        self.actUserDelete    = KAction(self, "user_delete",    "&Delete...",                   self.handleUserDelete, "Delete the selected user.")
        self.actExtrasSync    = KAction(self, "extras_sync",    "&Sync with LDAP directory...", None,                  "Synchronize the whole user list with the users of an LDAP server.")
        self.actExtrasOptions = KAction(self, "extras_options", "&Options...",                  None,                  "Set user management options for the whole Kumula suite.")

        # create Menu Structure
        self.menuUser = self.makePopupMenu()
        self.actUserEdit.addToMenuBar(self.menuUser)
        self.menuUser.insertSeparator()
        self.actUserNew.addToMenuBar(self.menuUser)
        self.actUserDelete.addToMenuBar(self.menuUser)
        self.MenuBar.insertItem(QString("&User"), self.menuUser, 1)

        self.menuExtras = self.makePopupMenu()
        self.actExtrasSync.addToMenuBar(self.menuExtras)
        self.menuExtras.insertSeparator()
        self.actExtrasOptions.addToMenuBar(self.menuExtras)
        self.MenuBar.insertItem(QString("E&xtras"), self.menuExtras, 2)


    def createToolBar(self):
        KKumulaMainWindow.createToolBar(self)
        self.actUserNew.addToToolBar(self.ToolBar, "New user")
        self.actUserEdit.addToToolBar(self.ToolBar, "Edit user")
        self.actUserDelete.addToToolBar(self.ToolBar, "Delete user")



    def __tr(self,s,c = None):
        return qApp.translate("MainWindow",s,c)


    def updateUserList(self, selectedUser=0):
        # save ID of the selected user, and if possible, select it after updating the list
        if (selectedUser == 0):
            if self.userList.currentItem():
                selectedUser = self.userList.currentItem().tag
            else:
                # this is when the program starts
                selectedUser = self.lastUser
        self.userList.clear()
        hasSelected = 0

        quUsers = kumula.DB.doSelect("users", ("id","username","employid","name","forename"), "username")
        while quUsers.next():
            myID       = quUsers.value(0).toInt()
            myUserName = quUsers.value(1).toString()
            myEmployID = quUsers.value(2).toString()
            mySureName = quUsers.value(3).toString()
            myForeName = quUsers.value(4).toString()
            item = KListViewItemWithID(self.userList, myID)
            item.setText(0, myUserName)
            item.setText(1, myEmployID)
            item.setText(2, mySureName)
            item.setText(3, myForeName)

            if (myUserName == selectedUser):
                self.userList.setSelected(item, 1)
                self.userList.setCurrentItem(item)
                hasSelected = 1

        if (hasSelected == 0):
            # always select at least one file
            if self.userList.firstChild():
                self.userList.setCurrentItem(self.userList.firstChild())
                self.userList.setSelected(self.userList.firstChild(), 1)
                self.actUserEdit.setEnabled(1)
                self.actUserDelete.setEnabled(1)
            else:
                # ...no user here...
                self.actUserEdit.setEnabled(0)
                self.actUserDelete.setEnabled(0)



    def slotUserListDoubleClick(self, aItem, aPoint, aColumn):
        print "slotUserListDoubleClick was called."
        if (aItem):
            self.handleUserEdit()



    def handleUserNew(self):
        dlg = KUserDialog(self)
        dlg.setCaption("Create new user")
        if (dlg.exec_loop() == QDialog.Accepted):
            quUsers = kumula.DB.doInsert("users", {"username":dlg.edUsername.text(),
                                                   "employid":dlg.edEmployID.text(),
                                                   "name":dlg.edSurename.text(),
                                                   "forename":dlg.edForename.text()})
            myUser = kumula.DB.lastAutoID()
            self.updateUserList(myUser)


    def handleUserEdit(self):
        myItem = self.userList.currentItem()
        dlg = KUserDialog(self)
        dlg.setCaption("Change user data")
        dlg.edUsername.setText(myItem.text(0))
        dlg.edEmployID.setText(myItem.text(1))
        dlg.edSurename.setText(myItem.text(2))
        dlg.edForename.setText(myItem.text(3))
        dlg.edEmployID.setEnabled(0)

        if (dlg.exec_loop() == QDialog.Accepted):
            quUsers = kumula.DB.doUpdate("users", {"username":dlg.edUsername.text(),
                                                   "employid":dlg.edEmployID.text(),
                                                   "name":dlg.edSurename.text(),
                                                   "forename":dlg.edForename.text()}, {"id":"%i" % myItem.tag})
            self.updateUserList(myItem.tag)



    def handleUserDelete(self):
        myItem = self.userList.currentItem()
        myQuestion = QString("Do you really want to<br>delete this user?<br><br><b>")
        myQuestion.append(myItem.text(3))
        myQuestion.append(" ")
        myQuestion.append(myItem.text(2))
        myQuestion.append("</b><br><br>Warning! Some relations to files,<br>documents and clients could get lost!")
        result = QMessageBox.warning(kumula.MainWin, "Question", myQuestion, "&Delete", "&Cancel") # , None, 0, 1)
        if (result == 0):
            quUsers = kumula.DB.doDelete("users", {"employid":myItem.text(1)}, 1)
            self.updateUserList()




    def handleExtrasSync(self):
        pass


